# first.cl -- task to subtract dark & add header params to brand new raw images
#
# 1. subtract dark
# 2. hedit image headers, adding: (basically this task is hadd.cl plus #0)
# 	1. BPM,BPMASK = bad pixel mask name
#	2. coadds   = number of internal coadds
# 	3. exptime  = (itime: exposure time per coadd) * (# of coadds)
# 	4. obs-date = date-obs but in dd/mm/yy format
# 	5. ut       = time-obs (or time-obs + 7)
# 	6. date-obs = yyyy-mm-dd format (old date-obs saved as dateorig)
# (following are optional)
# 	7. epoch, ra, dec = entered as parameters, if desired
# 	8. st = sidereal time, calculated from ut & date using astutil.asttimes
# 		& the observatory parameter (kpno, mtbigelow, mmto, ctio)
# 	9. airmass  = calculated from epoch, ra, dec, st, and obs-date
#
# 010315: created by PBH
# 010410: cleaned up "header keyword not found" errors 
#
# **bug: airmass+ airauto+ option assumes you're observing from MST time zone!
#	...but that's only for the "asttimes" task, which may be irrelevant
# **should be more options for computing ut than just time-obs or time-obs+7h

procedure c40ircfirst(imagelist,darksub,badmask)

string imagelist  {prompt="List of images to process (do not use an `@')"}
bool   darksub    {prompt="Subtract off dark frame?"}
string badmask    {prompt="Name of bad pixel mask to add to image header"}

string dark       {prompt="Name of the dark frame to subtract", mode="q"}
string darkstring {prompt="String to append to dark-subtracted images",mode="q"}
bool   coords=yes  {prompt="Add RA, DEC, and EPOCH to image header?"}
bool   airmass=yes {prompt="Enter airmass into image header?"}
bool   airauto=yes {prompt="Find airmass of image automatically?"}
real   airval     {prompt="Airmass of image", mode="q"}
string epoch	  {prompt="Epoch of coordinates"}
string ra	  {prompt="Right Ascension (surround with quotes)"}
string dec	  {prompt="Declination (surround w/quotes; use - but not +"}
string observatory="lco" {prompt="Observatory where observations were made"}
bool   ut=yes      {prompt="Is 'time-obs' in header ut or not?"}
bool   ddmmyy=no   {prompt="Is 'date-obs' in day/month/year format?"}
bool   echog=yes   {prompt="Beep when task is done?"}
struct *list

begin

string ddark, ddarkstring, bbadmask, check, base, iimagelist, ext
string darklist, flat, flatbase, meanfile, currentflat, tempflat, name
string editlist, current, obsdate, rra, ddec, eepoch, oobservatory, minsec
string obstime, timeobs, dateobs, century, utdate
real intime, aairval, ttime, mean, meanbad, x
int ncoadd, ddy, hhr, hhour, mmonth, yyear, dday, uttime, last2digits
bool ddarksub, ccoords, aairauto, uut, aairmass

iimagelist = imagelist
list = iimagelist
editlist = iimagelist
ddarksub = darksub
if (ddarksub) {
	ddark = dark
	ddarkstring = darkstring
	darklist = iimagelist//ddarkstring
}
bbadmask = badmask
ccoords = coords
aairmass = airmass
if (aairmass) {
	aairauto = airauto
#	if(!aairauto)aairval = airval
	if(!defpac("astutil")) {
		beep
  		print ("WARNING!!  Package ''astutil'' is not loaded; cannot set airmass in images.")
  		print ("Load the package and rerun the task.")
		return
		}
}
eepoch = epoch
rra = ra
ddec = dec
oobservatory = observatory
uut = ut

# Add BPM and other goodies to the image headers, just like in the hadd.cl task
print("===> Adding various parameters to the image headers...")
list = iimagelist
while (fscan (list, current) !=EOF) {
  hedit(current,"BPM",bbadmask,add+,ver-,show+,update+)
  hedit(current,"BPMASK",bbadmask,add+,ver-,show+,update+)
# Set exptime=itime * (# of coadds)
  ncoadd=1
  hedit(current,"coadds",(ncoadd),add+,ver-,show+,update+)
#
# Add colons to UT date in header
  imgets(current,"ut",>&"dev$null")
  utdate=substr(imgets.value,1,2)//":"//substr(imgets.value,4,5)//":"//substr(imgets.value,7,8)
  hedit(current,"ut",utdate,add+,ver-,show+,update+)
#
# The 'date-obs' keyword comes in either the old format "dd/mm/yy",
# the erroneous format "mm/dd/yy", or the new format "yyyy-mm-dd".
# Create new 'date-obs' keyword in the new format "yyyy-mm-dd".
  imgets(current,"dateorig",>&"dev$null")
  if (imgets.value!="0") {
    print ("DATEORIG keyword already exists -- first already run on image?")
  } else {
    imgets(current,"date-obs",>&"dev$null")
# Copy 'date-obs' keyword to 'dateorig' in original format.
    hedit(current,"dateorig",imgets.value,add+,ver-,show+,update+)
# Get yyyy-mm-dd from "date" keyword
    imgets(current,"date",>&"dev$null")
# Create header keyword 'obs-date', always in "dd/mm/yy" format.
# yyyy-mm-dd format:
#  1234567890
    obsdate=substr(imgets.value,9,10)//"/"//substr(imgets.value,6,7)//"/"//substr(imgets.value,3,4)
    hedit(current,"obs-date",obsdate,add+,ver-,show+,update+)
# Create new 'date-obs' in new format "yyyy-mm-dd" from 'obs-date' "dd/mm/yy"
#								    12345678
    dateobs=substr(imgets.value,1,10)
    hedit(current,"date-obs",dateobs,add+,ver-,show+,update+)
#
  }
}

if(ccoords){
	list = iimagelist
	while (fscan (list, current) !=EOF) {
  		hedit(current,"epoch",eepoch,add+,ver-,show+,update+)
  		hedit(current,"ra",rra,add+,ver-,show-,update+)
  		hedit(current,"ra",rra,add+,ver-,show+,update+)
  		hedit(current,"dec",ddec,add+,ver-,show-,update+)
  		hedit(current,"dec",ddec,add+,ver-,show+,update+)
#hedit test ra "(str('21:51:59.0'))" add+ ver- show-
	}
}

if(aairmass){
  if(aairauto){
    list = iimagelist
    while (fscan (list, current) !=EOF) {
  	imgets(current,"obs-date",>&"dev$null")
  	dday=int(substr(imgets.value,1,2))
  	mmonth=int(substr(imgets.value,4,5))
#  	yyear=int("19"//substr(imgets.value,7,8))
	last2digits=int(substr(imgets.value,7,8))
	if(last2digits <= 80) {
  	  yyear=int("20"//substr(imgets.value,7,8))
	} else {
  	  yyear=int("19"//substr(imgets.value,7,8))
	}
  	imgets(current,"ut",>&"dev$null")
  	hhr=int(substr(imgets.value,1,2))
  	minsec=substr(imgets.value,3,8)
	hhour=hhr-7
	if(hhour<0){
		dday=dday-1
		hhour=hhour+24.
		}
	if(dday<=0){
		mmonth=mmonth-1
# this will not always be totally accurate, and may fail for feb., but hey.
		dday=30
		}
# observing on new year's eve will require a new correction!
	ttime=real(str(int(hhour))//minsec)
	asttimes(observatory=oobservatory, year=yyear, month=mmonth, day=dday, time=ttime)
   	hedit(current,"st",asttimes.lmst,add+,ver-,show-,update+)
   	hedit(current,"st",asttimes.lmst,add+,ver-,show+,update+)
	setairmass(current, observatory=oobservatory, intype="middle", outtype="effective", date="obs-date",exposure="exptime", utmiddle="utmiddle", show+, update+, override+)
    }
  }
  else {
	list = iimagelist 
	while (fscan (list, current) !=EOF) { 
		print("Setting airmass for image "//current)
		hedit(current,"airmass",airval,add+,ver-,show+,update+)
		}
  }
}

# Subtract off Dark Current 
if (ddarksub) {
	list = iimagelist 
# if darklist doesn't exist, assume dsub imgs don't either; create it & dsubs
	if(!access(darklist)) {
	    print("===> Subtracting off Dark Current...")
	    while (fscan (list, name) !=EOF) {
		fileroot(name,validim+)
		base=fileroot.root
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		print(base//ddarkstring//ext, >> darklist)
  		imgets(name,"DARKSUB",>&"dev$null")
		if (imgets.value=="0") {
		    imarith(name, "-", ddark, base//ddarkstring//ext, verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",ddark,add+,ver-,show+,update+)
		} else {
      print("DARKSUB keyword exists; first already run? "//name//" just copied")
		    imcopy(name, base//ddarkstring//ext, verbose-)
		}
	    }
	} else {
	    print("Darksubbed image list "//darklist//" already exists; images assumed to exist too.")
	}
#imarith("@"//iimagelist, "-", ddark, "@"//darklist, verbose-)
#hedit("@"//darklist,"DARKSUB",ddark,add+,ver-,show+,update+)
print(" ") 
print("===> Dark-subtracted images are listed in the file "//darklist//".") 
}

if(echog)beep

end

