procedure crzap2(inlist,outlist)

#  Cosmic ray cleaning routine using imexam and fixpix
#
#  970227: created from dimsum.xzap (original courtesy Mark Dickinson)
#  980428: rewrote to call fixpix and text2mask from proto or nmisc,
#	   altho nmisc.fixpix is a bit more versatile & would be preferred
#
#  Calls scripts fileroot.cl, text2mask and fixpix

string	inlist	    {prompt="Image(s) for cosmic ray cleaning"}
string	outlist	    {prompt="Output image(s)"}
bool    cleanup     {yes, prompt="Delete CR mask & logfile after execution?"}
bool    overwrite   {no, prompt="Overwrite input image with output image?"}
bool    echog       {yes, prompt="Beep when task is done?"}

struct	*inimglist
struct	*outimglist

begin

	real    skysig,skymode,crthresh,objthresh,negthresh,dmin,dmax
        int     nbox,nbox2,nin,x,y
        string  infile, outfile, img, outimg, statfile, objmask
	bool	maskobj, maskneg, go

go=yes
#if(!defpac("nmisc")) {go=no}
if(!deftask("fixpix")) {go=no}
if(!deftask("text2mask")) {go=no}
if(go) {

# Expand file lists into temporary files.
	infile =  mktemp("tmp$zap")
	outfile = mktemp("tmp$zap")
	sections (inlist,option="fullname",>infile)
	nin = sections.nimages
	sections (outlist,option="fullname",>outfile)
	if (nin != sections.nimages) {
	print ("ERROR:   Numbers of input and output images do not match.")
		return
	}
	inimglist = infile
	outimglist = outfile

# Loop through input files:
	while (fscan(inimglist,img) != EOF) {

# Strip extension off input file name.
		fileroot (img,validim+)
		img = fileroot.root

# Read name of output file.
		if (fscan(outimglist,outimg) == EOF) {
		print ("ERROR:   Problem scanning output file name in crzap.")
			return
		}
		fileroot (outimg,validim+)
		outimg = fileroot.root

# Imexam
print ("Identify rectangular cosmic ray regions by typing `b' at the corners.")
	imexam (img, frame=1, logfile="_crlog."//img, keeplog+)
	vi("_crlog."//img)
	imgets (img, "i_naxis1")
	x=int(imgets.value)
	imgets (img, "i_naxis2")
	y=int(imgets.value)
# Create mask
	print("text2mask _crlog."//img, "_crmask2_"//img//".pl", "ncols="//x, "nlines="//y, "linterp=1, cinterp=2, square=2, pixel=2", >> "dev$null")
	text2mask ("_crlog."//img, "_crmask2_"//img//".pl", ncols=x, nlines=y, linterp=1, cinterp=2, square=2, pixel=2)
# following for nmisc.fixpix only
#	text2mask ("_crlog."//img, "_crmask2_"//img//".pl", ncols=x, nlines=y, linterp=1, cinterp=2, square=3, pixel=3)
# Fix image
	if(outimg!=img) {
		imcopy (img, outimg, ver-)
	} else {
		if(!overwrite) {
			imcopy (img,"_orig."//img,ver-)
		  print("Original image "//img//" will be saved as _orig."//img)
		}
	}
fixpix (outimg, "_crmask2_"//img//".pl", linterp=1, cinterp=2)
# following for nmisc.fixpix only
#fixpix (outimg, "_crmask2_"//img//".pl", linterp=1, cinterp=2, ninterp=3)
# Next input file
	}

# Clean up.
		if (cleanup) delete ("_crmask2_"//img//".pl",ve-)
		if (cleanup) delete ("_crlog."//img,ve-)

	inimglist = ""; delete (infile, verify-)
	outimglist = ""; delete (outfile, verify-)

} else {
	print ("WARNING! Tasks 'fixpix' and text2mask not loaded -- crzap2 can't run!")
	print ("To access tasks load 'nmisc' for V2.10 'proto' for V2.11.")
}

if(echog)beep

end
