# irimfirst.cl -- add header params to raw IRIM (4-m) images and optionally do
#   nonlinearity correction, saturated pixel replacement, & dark subtraction
#
# 1. do nonlinearity correction & add NLCOR (& poss. SATVAL) header card(s)
# 2. subtract dark & add DARKSUB header card
# 3. hedit image headers, adding: (basically this is task 'hadd.cl', except #0)
# 	0. BPM,BPMASK = bad pixel mask name
# 	1. exptime  = (itime: exposure time per coadd) * (# of coadds)
# 	2. obs-date = date-obs (obs-date should have dd/mm/yy format)
#
# 941217: adapted for 4-m from first.cl
# 950328: added DARKSUB header card insertion and nonlinearity option
# 950403: changed correction to account for nonlinearity before first read
# 960203: changed to allow suffix=""
# 970321: changed to yield obs-date in dd/mm/yy format 
# 980414: added compatibility to incoming date-obs in new-millenium format and
#   output of date-obs in same format (valid for data from 1990 to 2089);
#   original date-obs is output as dateorig
# 990320: fixed bug where exptime & itime were feedbacking on each other
# 990427: DARKSUB keyword added to darksubbed images, not original ones
# 030206: can't dark-subtract in place; suffix="" && darksub=yes won't darksub
#
# **cannot replace saturated pixels except at same time as nonlin. correction

procedure irimfirst(inlist,nonlin,darksub)

string inlist  {prompt="List of images to process (do not use an `@')"}
bool   nonlin  {prompt="Do nonlinearity correction?"}
bool   darksub {prompt="Subtract off dark frame?"}

string suffix  {prompt="String to be appended to output images",mode="q"}
real   slope   {min=1.,prompt="Slope coefficient of nonlinearity correction",mode="q"}
string intname {prompt="Name of integration time header keyword",mode="q"}
bool   replace {prompt="Replace `saturated' pixels?",mode="q"}
real   maxval  {40000.,min=30000.,prompt="Maximum good raw data value",mode="q"}
string dark    {prompt="Name of the dark frame to subtract",mode="q"}
string badmask {prompt="Name of bad pixel mask to add to image header"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *list

begin

string ddark, ssuffix, bbadmask, check, base, ilist
string olist, flat, flatbase, meanfile, currentflat, tempflat, name
string editlist, current, obsdate, rra, ddec, eepoch, oobservatory, minsec
string obstime, timeobs, dateobs, century
real   aairval, ttime, mean, meanbad, x
int    ncoadd, ddy, hhr, hhour, mmonth, dday, uttime
bool   ddarksub, ccoords, uut, nnonlin
string  iintname        # equals intname
string  instring        # temporary string
real    sslope          # equals slope of nonlinearity correction
real    mmaxval         # equals maxval
real    valmax          # equals maxval
string  img             # fscanned name of individual input image
string  oimg            # fscanned name of individual output image
real    intime          # equals int_time
real    delaytime       # delay time between reset & first read
real    intfactor       # factor for use in nonlinearity correction
bool    rreplace        # equals replace
bool    go              # just do it
bool    goon            # just do it
struct  theadline       # temporary struct for header information

ilist = inlist
editlist = ilist	# editlist lists images currently being worked on
olist = ilist		# olist lists output images
nnonlin = nonlin
ddarksub = darksub
ssuffix = suffix
if(nnonlin){
# If suffix=="", insert temporary suffix to avoid overwriting
	if(ssuffix==""){
		ssuffix="filthpig"
		beep
		print("WARNING! Input files will be overwritten!")
	}
	olist = ilist//ssuffix  # olist lists output images
        sslope = slope
        iintname = intname
        rreplace = replace
        if(rreplace){
		mmaxval = maxval
		valmax = mmaxval
	}
}
if(ddarksub){
	if(!nnonlin){
		ssuffix = suffix
		olist = ilist//ssuffix  # olist lists output images
	}
	ddark = dark
}
bbadmask = badmask

# The "go" parameter won't let the task run if the needed tasks aren't loaded.
go=yes
if(!deftask("fileroot")) {go=no}
if(!deftask("imcalc")) {go=no}
if(go) {

# Do nonlinearity correction
goon=no
if(nnonlin){
	print("===> Running nonlinearity correction procedure...")
	list = ilist
	editlist = olist
	while (fscan(list,img) != EOF) {
        	fileroot(img,validim+)
        	oimg=fileroot.root//ssuffix//"."//fileroot.extension
# add output image name to output image list unless suffix=""
#        	if(ssuffix!="filthpig")print(oimg, >> olist)
        	print(oimg, >> olist)
# do the correction only if input image has not already been corrected
		imgets(img,"NLCOR",>&"dev$null")
		if(imgets.value=="0")goon=yes
# if output image exists...
		if(access(oimg)){
			imgets(oimg,"NLCOR",>&"dev$null")
# ...and has not been nlcorrected, delete it 
			if(imgets.value=="0"){
				imdel(oimg,ver-)
			}
# ...if it has been nlcorrected, print error message & do not do the correction
			else{
				goon=no
       print("Nonlinearity correction already done on "//oimg//"-- not redone.")
			}
		}
		if(goon){
        		imgets(img,iintname)
        		intime=real(imgets.value)
# Find delay time...
			if(intime>=1.381){
        			delaytime=1.
#        			intfactor=1.+(1./intime)
			}
			else{
				delaytime=intime-0.381
#				intfactor=2.-(0.381/intime)
			}
			intfactor=1.+((2.*delaytime)/intime)
# Do nonlinearity correction.  The correction is approximate, but should be
# good for pixels requiring <=10% correction, above which they saturate anyway.
			imcalc(img,oimg,"(im1)/(1.-im1*("//intfactor//"/"//sslope//"))", pixtype="real",nullval=0.,ver-)
#			imcalc(img,oimg,"("//sslope//"*im1)/("//sslope//"-im1*"//intfactor//")",pixtype="real",nullval=0.,ver-)
			hedit(oimg,"NLCOR",sslope,add+,ver-,show+,update+)
# Replace "saturated" pixels if desired
        		if(rreplace){
                	    mmaxval=(sslope*valmax)/(sslope-valmax*intfactor)
                	    imreplace(oimg,mmaxval,lower=mmaxval,upper=INDEF)
			    hedit(oimg,"SATVAL",mmaxval,add+,ver-,show+,update+)
			}
        	}
# If correction already done on input image, do not redo, just copy to output
		else{
	print("Nonlinearity correction already done on "//img//"-- not redone.")
			if(!access(oimg))imcopy(img,oimg,ver-)
		}
	}
}

# Subtract off Dark Current 
if (ddarksub) {
	print("===> Subtracting off Dark Current...")
# If final output list is not same as current list, need to create output list
	if(olist!=editlist){
# if olist doesn't exist, assume darksub'd imgs don't either; create it & dsubs
	    if(!access(olist)) {
	    	print("===> Subtracting off Dark Current...")
		list=editlist
		while (fscan(list,img) != EOF) {
        		fileroot(img,validim+)
        		oimg=fileroot.root//ssuffix
        	    if(fileroot.extension!="")oimg=oimg//"."//fileroot.extension
        		print(oimg, >> olist)
  			imgets(img,"DARKSUB")
			if (imgets.value=="0") {
		    		imarith(img, "-", ddark, oimg, verbose-)
		                hedit(oimg,"DARKSUB",ddark,add+,ver-,show+,upd+)
			} else {
  print("DARKSUB keyword exists; irimfirst already run? "//name//" just copied")
		    		imcopy(img, oimg, verbose-)
			}
	    	}
	    }
	} else {
	    print("Darksubbed image list "//olist//" already exists; images assumed to exist too.")
	}
	print(" ") 
        print("===> Dark-subtracted images are listed in the file "//olist//".")
#	imarith("@"//editlist, "-", ddark, "@"//olist, verbose-)
#	hedit("@"//olist,"DARKSUB",ddark,add+,ver-,show+,update+)
	editlist=olist
}

# Add the BPM to the image headers
print("===> Adding various parameters to the image headers...")
hedit("@"//editlist,"BPM",bbadmask,add+,ver-,show-,update+)
hedit("@"//editlist,"BPMASK",bbadmask,add+,ver-,show-,update+)

# Add other goodies to the image headers, just like in the hadd.cl task
list = editlist
while (fscan (list, current) !=EOF) {
  imgets(current,"int_s")
  intime=real(imgets.value)
  imgets(current,"coadds")
  ncoadd=real(imgets.value)
  hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
  imgets(current,"date-obs")
  hedit(current,"dateorig",(imgets.value),add+,ver-,show+,update+)
# date-obs is either in dd/mm/yy format or yyyy-mm-dd format
  if(substr(imgets.value,3,3)=="/") {
# if input date-obs is in dd/mm/yy format, rework to get output date-obs
    if ((substr(imgets.value,7,7))=="9") {
      century=19
    } else {
      century=20
    }
    dateobs=century//substr(imgets.value,7,8)//"-"//substr(imgets.value,4,5)//"-"//substr(imgets.value,1,2)
    hedit(current,"date-obs",dateobs,add+,ver-,show+,update+)
# if input date-obs is in dd/mm/yy format, obs-date is a direct copy of it
    hedit(current,"obs-date",(imgets.value),add+,ver-,show+,update+)
  } else {
# if input date-obs is in yyyy-mm-dd format, rework to get obs-date 
    obsdate=substr(imgets.value,9,10)//"/"//substr(imgets.value,6,7)//"/"//substr(imgets.value,3,4)
    hedit(current,"obs-date",obsdate,add+,ver-,show+,update+)
  }
}

# If suffix=="", 
if(ssuffix=="filthpig"){
  imdel("@"//ilist,ver-)
  imren("@"//olist,"@"//ilist,ver-)
  del(olist,ver-)
  olist=ilist
}

print(" ") 
print("===> Processed images are listed in the file "//olist//".")
if(ddarksub || nnonlin){
	print("===> You should rerun the task with nonlin=no and/or darksub=no to properly")
	print("===> update the original image headers, if you plan to use those images.")
}

}

else {
	print("WARNING: tasks fileroot and/or stsdas.toolbox.imgtools.imcalc")
	print("are not loaded!  Please load them and restart this task.")
        }

if(echog)beep

end

