# irrotate -- rotate and resample images and recalculate shifts for them
#
# Calls scripts fileroot, filecalc, addcomment.
#
# Output:
# "*br"//mag//ssuffix//".imh"		output rotated, resampled images
# inputlist//"br"//mag//ssuffix		list of above images
# shiftexactlist//"br"//mag//suffix	list of shifts for above images
# inputlist//"br"//mag//ssuffix//"pl"	list of new bad pixel images created
#
# 960307: created from ircoadd2.cl
# 960402: add header keyword ROTATED = angle//"x"//magfac
# 960817: allowed for non-creation of shifts file if desired
# 970426: added rotation of TPLIMG images
# 970505: fixed bug -- need boundary=nearest for rotation of TPLIMG images
# 970513: fixed bug which made offsets -0.5 < offsets < 0.5 round to 1. not 0
# 970630: fixed minv bugs, nothing major
# 990516: use imcalc instead of imreplace
#
# **should include optional tpl division, before bp zeroing

procedure irrotate(rotlist,suffix,angle,scale,magfac,shiftexactlist,logfile)

string rotlist	{prompt="List of images to rotate"}
string suffix 	{prompt="UNIQUE suffix to append to rotated images"}
real angle   	{prompt="Rotation angle in degrees"}
real scale      {prompt="Input image scale compared to reference image"}
int  magfac  	{2,min=1,max=10,prompt="Output image magnification factor (subpix shifts)"}
string shiftexactlist=""  {prompt="File with EXACT image shifts"}
string logfile  {prompt="Name of log file"}

string i_interp   {"linear",enum="nearest|linear|poly3|poly5|spline3",prompt="IMLINTRAN: Interpolation type"}
string i_boundary {"constant",enum="nearest|constant|reflect|wrap",prompt="IMLINTRAN: Boundary extension type"}
real   i_constant {0.,prompt="IMLINTRAN: Value for constant boundary"}
bool   bpzero=no  {prompt="Zero images by bad pixel masks before rotating?"}
bool   newmask=no {prompt="Override bad pixel masks listed in image headers?"}
string badmask 	  {prompt="Bad pixel mask to use instead",mode="q"}
bool   invert=no  {prompt="Invert output bad pixel masks so bad pixels=0?"}
real   bpfrac     {0.75,min=0.,max=0.99,prompt="Threshold bad pixel value in the rotated bp mask"}
bool   norot=no   {prompt="Skip image rotation and calculate shifts only?"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *list

begin

string sshiftfraclist, iinterp, ssuffix, sshiftlist, subimg, explist, bpmimg
string rrotlist, bbadmask, sshiftoutlist, ooutimage, zzerofile, sstackimage
string llogfile, check, zzeroname, sscalefile, sscalename, img, eexpname, blk
string listimshift, mmoreword, wwordzero, mmorename, subimgold, expmaskold, ext
string rrotlist1, rrotlist2, sshiftexactlist, newzzeroname, image, newbp, sect
string oldbp, holesmask, tmpstring, s1, s2, s3, bprotlist, o_interp, o_boundary
string oldtpl, newtpl
real avg, x, sig, invmag, valmax, bbpfrac, outfactor, invfactor, aangle
real effairmass, wtavg, nncombine, exp, exposure, sf, zf, testold, eexptime
real xold, yold, xnew, ynew, angrad, x_shift, y_shift, sscale, tmpreal, y, z
real o_constant
int mag, mag2, xlength, ylength, j, k, n_cols, n_lines
int xmin, ymin, newx, newy, oldx, oldy, test1, test1old, test2, test2old
int sstartno, eendno, i, numpix, num, xdim, ydim, newxsize, newysize, ncoadds
bool mmakescalefile, sstack, mmakestack, mmorescale, go, mmorehow, pputzero
bool nnewmask, sstackdel, sstdstar, ssubpixscale, noappend, iinvert, bbpzero
struct theadline

	rrotlist = rotlist
		rrotlist1 = rrotlist
	ssuffix=suffix
		if(ssuffix==""){
			date | scan(s1,s2,i,x,s3,k)
			y=int(100*x)
			ssuffix="rot"//i//s2//y//"_"
			print("Auto-generated image suffix is "//ssuffix)
		}
	aangle=angle
		angrad=aangle*3.1415926535/180.
	sscale = scale
	mag = magfac
		invmag=1./mag
		invfactor=sscale*invmag
		mag2=mag*mag
		blk = "br"//mag
		rrotlist2 = rrotlist//blk//ssuffix
		bprotlist = rrotlist//blk//ssuffix//"pl"
	sshiftexactlist = shiftexactlist
		sshiftoutlist=sshiftexactlist//blk//ssuffix
		#sshiftoutlist = sshiftexactlist//blk
	llogfile = logfile

	o_interp = i_interp
	o_boundary = i_boundary
		o_constant=0.
		if(o_boundary=="constant")o_constant=i_constant
	bbpzero = bpzero
	nnewmask = newmask
		if(nnewmask)bbadmask=badmask
        iinvert = invert
	bbpfrac = bpfrac

# Certain tasks/packages must be loaded for task to run.
go=yes
if(!deftask("fileroot")) {go=no}
if(!deftask("filecalc")) {go=no}
if(!defpac("imgtools")) {go=no}
if(go) {

# Set up log file, overwriting if it already exists.
if(llogfile == "") llogfile="log."//ooutimage
print("", >> llogfile)
print("===> PHIIRS.IRROTATE:", >> llogfile)
print("===> Input images from "//rrotlist1//" rotated by angle "//aangle, >> llogfile)
if(bbpzero)print("===> (after having all bad pixels zeroed), then", >> llogfile)
print("===> rescaled by factor "//sscale//" to match reference image,", >> llogfile)
print("===> and resampled by factor "//mag//" in each direction.", >> llogfile)
print("===> New x-y shifts for the output images listed in "//rrotlist2, >> llogfile)
print("===> can be found in "//sshiftoutlist, >> llogfile)

# BP ZEROING IF DESIRED
if (bbpzero) {
	print("Zeroing images in "//rrotlist1//" by their bad pixel masks.")
	list=rrotlist1
	while (fscan (list, img) !=EOF) {
    		fileroot(img,validim+)
		image=fileroot.root
		imgets(image,"BPM",>&"dev$null")
		bpmimg=str(imgets.value)
	     imar(img,"*",bpmimg,img,pixtype="real",calctype="real",noact-,ver-)
	}
}

# ROTATION 
# If output image list already exists, do not append to it.
if (!norot) {
noappend=no
if (access(rrotlist2)) {
	noappend=yes
	beep
	print("WARNING! Output image list "//rrotlist2//" already exists.")
	print("Any images listed therein will be created if they do not exist,")
	print("but no image names will be appended to this preexisting file.")
}
print("===) Rotating images and expanding by factor "//mag//" in both directions...")
list=rrotlist1
while (fscan (list, img) !=EOF) {
    fileroot(img,validim+)
    image=fileroot.root
    ext=fileroot.extension
    if(ext==""){
		ext=".imh"
	} else {
		ext="."//ext
	}
    imgets(image,"ROTATED",>&"dev$null")
    if(imgets.value=="0"){
	if(!noappend)print(image//blk//ssuffix, >> rrotlist2)
	imgets(image,"i_naxis1")
       	xlength=int(imgets.value)
       	imgets(image,"i_naxis2")
       	ylength=int(imgets.value)
	n_cols=int(xlength/invfactor)
	n_lines=int(ylength/invfactor)
# Increasing output sizes by 1 does preserve all info, but makes coadding with
# nonrotated images painful.
#	n_cols=int(xlength/invfactor)+1
#	n_lines=int(ylength/invfactor)+1
# If output rotated & resampled image does not exist, create it
	if(!access(image//blk//ssuffix//ext)){
		imlintran(image, image//blk//ssuffix, xrotation=aangle, yrotation=aangle, xmag=invfactor, ymag=invfactor, xin=INDEF, yin=INDEF, xout=INDEF, yout=INDEF, ncols=n_cols, nlines=n_lines, interpolant=o_interp, boundary=o_boundary, constant=o_constant, fluxconserve+, nxblock=512, nyblock=512)
#		imlintran(image, image//blk//ssuffix, xrotation=aangle, 
#yrotation=aangle, xmag=invfactor, ymag=invfactor, xin=INDEF, yin=INDEF, 
#xout=INDEF, yout=INDEF, ncols=n_cols, nlines=n_lines, interpolant="linear", 
#boundary="constant", constant=0., fluxconserve+, nxblock=512, nyblock=512)
        	imgets(image//blk//ssuffix,"TPLIMG",>&"dev$null")
                if(imgets.value!="0")hedit(image//blk//ssuffix,"TPORIG",imgets.value,add+,ver-,show-,update+)
        	imgets(image//blk//ssuffix,"BPMASK",>&"dev$null")
             if(imgets.value=="0")imgets(image//blk//ssuffix,"BPM",>&"dev$null")
	hedit(image//blk//ssuffix,"BPORIG",imgets.value,add+,ver-,show-,update+)
	} else {
# If output image already exists, only print a warning.
print("WARNING! "//image//blk//ssuffix//" already exists; not overwritten")
	}
# imlintran the TPL image, if not already done.  
        imgets(image//blk//ssuffix,"TPORIG",>&"dev$null")
        if(imgets.value=="0"){
	    print("Image "//image//blk//ssuffix//" is missing TPORIG keyword!")
	} else {
	    fileroot(imgets.value,validim+)
	    oldtpl=fileroot.root
	    newtpl=oldtpl//blk//ssuffix
	    if(!access(oldtpl//blk//ssuffix//ext)){
		imlintran(oldtpl, oldtpl//blk//ssuffix//ext, xrotation=aangle, yrotation=aangle, xmag=invfactor, ymag=invfactor, xin=INDEF, yin=INDEF, xout=INDEF, yout=INDEF, ncols=n_cols, nlines=n_lines, interpolant=o_interp, boundary="nearest", constant=o_constant, fluxconserve-, nxblock=512, nyblock=512)
	    }
	hedit(image//blk//ssuffix,"TPLIMG",oldtpl//blk//ssuffix//ext,add+,ver-,show-,update+)
	}
# imlintran the bad pixel mask, if not already done.  Include "edge effects".
        imgets(image//blk//ssuffix,"BPORIG",>&"dev$null")
        if(imgets.value=="0"){
	    beep
	    print("Image "//image//blk//ssuffix//" is missing BPORIG keyword!")
	} else {
	    fileroot(imgets.value,validim+)
	    oldbp=fileroot.root
	    newbp=oldbp//blk//ssuffix
	    if(!access(oldbp//blk//ssuffix//".pl")){
		if(!iinvert){
		    imcopy(oldbp,oldbp//"tmp"//ext,ver-)
                } else {
		    minv(oldbp,oldbp//"tmp"//ext)
                }
		imar(oldbp//"tmp"//ext,"/",oldbp//"tmp"//ext,oldbp//"tmpe"//ext, divzero=0., pixtype="real", calctype="real",ver-)
		imlintran(oldbp//"tmp"//ext//","//oldbp//"tmpe"//ext, newbp//"tmp"//ext//","//newbp//"tmpe"//ext, xrotation=aangle, yrotation=aangle, xmag=invfactor, ymag=invfactor, xin=INDEF, yin=INDEF, xout=INDEF, yout=INDEF, ncols=n_cols, nlines=n_lines, interpolant=o_interp, boundary=o_boundary, constant=o_constant, fluxconserve+, nxblock=512, nyblock=512)
#		imlintran(oldbp//"tmp"//ext//","//oldbp//"tmpe"//ext, 
#newbp//"tmp"//ext//","//newbp//"tmpe"//ext, xrotation=aangle, yrotation=aangle,
#xmag=invfactor, ymag=invfactor, xin=INDEF, yin=INDEF, xout=INDEF, yout=INDEF, 
#ncols=n_cols, nlines=n_lines, interpolant="linear", boundary="
#constant", constant=0., fluxconserve+, nxblock=512, nyblock=512)
# Replace edge pixels in the edge mask
		minmax(newbp//"tmpe"//ext,force+,ver-)
		valmax=minmax.maxval
		#imreplace(newbp//"tmpe"//ext,0.,lower=INDEF,upper=0.99*valmax)
		#imreplace(newbp//"tmpe"//ext,1.,lower=0.98*valmax,upper=INDEF)
		imcalc(newbp//"tmpe"//ext,newbp//"tmpe"//ext,"if im1 .lt. "//valmax//" then 0 else 1",ver-)
# Replace only slightly bad pixels in the bad pixel mask with 1.
		minmax(newbp//"tmp"//ext,force+,ver-)
		valmax=minmax.maxval
		#imreplace(newbp//"tmp",0.,lower=INDEF,upper=bbpfrac*valmax)
	        #imreplace(newbp//"tmp",1.,lower=0.9*bbpfrac*valmax,upper=INDEF)
		imcalc(newbp//"tmp"//ext,newbp//"tmp"//ext,"if im1 .le. "//bbpfrac*valmax//" then 0 else 1",ver-)
# Multiply bad pixel mask by edge mask and convert to .pl format
		imar(newbp//"tmp","*",newbp//"tmpe",newbp//"tmp",pixt="real",calct="real",ver-)
		if(!iinvert){
		    imcopy(newbp//"tmp"//ext,newbp//".pl",ver-)
                } else {
		    minv(newbp//"tmp"//ext,newbp//".pl")
                }
# Clean up
		imdel(oldbp//"tmp*"//ext,ver-)
		imdel(newbp//"tmp*"//ext,ver-)
		print(newbp//".pl", >> bprotlist)
	    }
# Insert the rotated & resampled bad pixel mask in the image header
	   hedit(image//blk//ssuffix,"BPM",newbp//".pl",add+,ver-,show-,update+)
	hedit(image//blk//ssuffix,"BPMASK",newbp//".pl",add+,ver-,show-,update+)
	}
#
# addcomment ROTATED w/angle, magfac
	hedit(image//blk//ssuffix,"ROTATED",str(aangle)//"x"//str(mag),add+,ver-,show-,update+)
#
# If image has already been rotated, do not rotate.
    } else {
        print("WARNING! Image "//image//" already rotated; not rotated again.")
    }
# Rotate/resample next image...
}
} else {
print("===> Image rotation not done.")
}

# CALCULATE SUBPIXEL SHIFTS
# Create subpixel shift file if desired and if not already in existence
if (sshiftexactlist!="" && !access(sshiftoutlist)){
	print("===> Calculating shifts for rotated, resampled images...")
	if (access("tmp"//sshiftoutlist))del("tmp"//sshiftoutlist,ver-)
	list=sshiftexactlist
	while (fscan (list,xold,yold) != EOF) {
		xnew=(mag/sscale)*(xold*cos(angrad) - yold*sin(angrad))
		ynew=(mag/sscale)*(yold*cos(angrad) + xold*sin(angrad))
		print (xnew, ynew, >> "tmp"//sshiftoutlist)
	}
# Round off to integer shifts (in units of new pixels, natch).
# Extensive code needed to make sure offsets -0.5 < offsets < 0.5 round to 0.
	list="tmp"//sshiftoutlist
        while(fscan(list,x,y) !=EOF) {
                if(((abs(x))>=0.5) && ((abs(y))>=0.5)) {
                        print(x,y, >> "temp."//sshiftoutlist)
 filecalc("temp."//sshiftoutlist,"$1;$2",format="%4.0f %4.0f", >> sshiftoutlist)
                        del ("temp."//sshiftoutlist, ver-, >& "dev$null")
                } else {
                        if((abs(x))<=0.5) {
                                x=0.
                        }
                        if((abs(y))<=0.5) {
                                y=0.
                        }
                        print(x,y, >> "temp."//sshiftoutlist)
 filecalc("temp."//sshiftoutlist,"$1;$2",format="%4.0f %4.0f", >> sshiftoutlist)
                        del ("temp."//sshiftoutlist, ver-, >& "dev$null")
                }
        }
	del("tmp"//sshiftoutlist,ver-,>&"dev$null")
	del("temp"//sshiftoutlist,ver-,>&"dev$null")
} else {
beep
print("WARNING! "//sshiftoutlist//" not created (may already exist).")
}

## Temporary fix -- correction of zero offsets
#if(!ssubpixscale && !access(zzeroname//blk)) {
#	filecalc(zzeroname,"$1/"//mag2,format="%16.12f", >> zzeroname//blk)
#	newzzeroname=zzeroname//blk
#	zzeroname=newzzeroname
#}

# Clean up.  (IRCOADD2 ONLY!)
# imdel("@"//rrotlist2, ver-)
# del(rrotlist2, ver-)
# imdel("@"//bprotlist, ver-)
# del(bprotlist, ver-)
# del(sshiftoutlist, ver-)

if(!norot)print("Images in "//rrotlist//" were rotated to make "//rrotlist2)
if(sshiftexactlist!="")print("Next run `ircoadd' w/shifts from "//sshiftoutlist)

}
else {
   print("WARNING: stsdas.imgtools package, filecalc, or fileroot not loaded!")
   print("Please load the appropriate package or task and restart this task.")
        }

print(" ")
if(echog)beep

end

