# irshift2 -- measure scale and rotation between two images' coord systems
#
# Task calls following other tasks: filecalc.
#
# Output:
# prefix.georefin.suffix	object coordinates in ref and input images
# prefix.geomapin.suffix	input file for geomap task
# 
# CHANGES:
# 960307: created from irshift.cl
# 980429: rewritten to work without "nlocal.linenum" task
# 980809: now suggests xmin xmax ymin ymax for immatch.geomap

procedure irshift2(refimage,inimage,prefix,suffix)

string refimage	      {prompt="Reference image"}
string inimage	      {prompt="Input image"}
string prefix	      {prompt="Prefix string for output files"}
string suffix	      {prompt="Suffix string for output files"}

real radius=5.	      {prompt="Imexamine aperture photometry radius"}
real buffer=10.	      {prompt="Imexamine sky buffer"}
real width=10.	      {prompt="Imexamine sky annulus width"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *list

begin

string rrefimage, fflatlist, rrefcoords, bbadmask, rroundname
string sshiftlist, coarsefile, ooffname, ssuffix, iintname, ffracname
string pprefix, logoffname, sl, sx, img, iinterp, bboundary
string iinimage, stringline1, stringline2, stringline1old, stringline2old
string stringline1tmp, stringline2tmp, georefin, geomapin, string1, string2
real p, pixscale, stcoordx, stcoordy, nrows, ncols, delx, dely, adelx, adely
real xold, yold, xxshiftold, yyshiftold, xshiftnew, yshiftnew, astcoordx
real astcoordy, nl, nx, x, y
int xref	# Reference object x coordinate (nearest integer)
int yref	# Reference object y coordinate (nearest integer)
int number, xtmp, ytmp, z, linenum, imax, imay
bool go

#The "go" parameter won't let the task run if you haven't loaded the ctio pkg.
go=yes
if(!deftask("filecalc")) {go=no}
if(go) {

rrefimage = refimage
iinimage = inimage
pprefix = prefix
ssuffix = suffix

rimexam.radius=radius
rimexam.buffer=buffer
rimexam.width=width

georefin = pprefix//".georefin."//ssuffix
geomapin = pprefix//".geomapin."//ssuffix

print(" ")
print("===) Choose an object that appears in both images, find its coordinates")
print("===) (type 'a') first in frame 1 and then in frame 2, and repeat with as many")
print("===) objects as desired.  The separation of coordinates from the different")
print("===> images will be done later, but **THE SEPARATION WILL FAIL** if the")
print("===> frame1/frame2 cycle is deviated from even slightly!  The imexam output")
print("===> log will be displayed for editing when done, in case mistakes were made.")
print(" ")

	display(iinimage,2)
	display(rrefimage,1)
	imexam(logfile=georefin, keep+)

print(" ")
print("===> In case mistakes were made, the coordinate file can now be edited.")
	vi(georefin)

print("===> Constructing geomap input file...")
	del("temp.joinfile*", ver-)
# copy the imexam output to a working file
	copy(georefin,"temp.joinfile",ver-)
# remove all comment lines
	!grep -v '^#' temp.joinfile > temp.joinfile1
# make sure there is an even number of lines remaining
	count("temp.joinfile1") | scan (nl)
	if((frac(nl/2.))!=0.){
		beep
		print ("ERROR!  Odd number of coordinates in this file!  Fix it!", >> "temp.joinfile1")
		vi("-c ':$'", "temp.joinfile1")
	}
# number the lines in the input file
#	linenum("temp.joinfile1","temp.joinfile0",first_line=1,increment=1)
	linenum=1
	list="temp.joinfile1"
	while(fscan(list, string1) !=EOF) {
		print(linenum, >> "temp.joinfile0")
		linenum=linenum+1
	}
# extract the first 3 columns in appropriate format
#	filecalc("temp.joinfile0","$1;$2;$3",lines="1-99999",format="%5.0f%9.2f%9.2f", >> "temp.joinfile2")
	filecalc("temp.joinfile0,temp.joinfile1","$1@1;$1@2;$2@2",lines="1-99999",format="%5.0f%9.2f%9.2f", >> "temp.joinfile2")
# initialize loop
	list="temp.joinfile2"
	stringline1=""
	stringline2=""
	stringline1old=""
	stringline2old=""
	stringline1tmp=""
	stringline2tmp=""
# create one string of odd line numbers "1,3,5,..." and one of even. 
# both of which will eventually be used as parameters for filecalc
	while(fscan(list, number, xtmp, ytmp) !=EOF) {
		if((frac(number/2.))!=0.){
			stringline1tmp=stringline1old//number//","
		} else {
			stringline2tmp=stringline2old//number//","
		}
		stringline1old=stringline1tmp
		stringline2old=stringline2tmp
	}
# cut off trailing comma
	z=strlen(stringline1tmp)-1
	stringline1=substr(stringline1tmp,1,z)
	z=strlen(stringline2tmp)-1
	stringline2=substr(stringline2tmp,1,z)
# extract coords from image 1
	filecalc("temp.joinfile1","$1;$2",lines=stringline1,format="%9.2f%9.2f", >> "temp.joinfile1a")
# extract coords from image 2
	filecalc("temp.joinfile1","$1;$2",lines=stringline2,format="%9.2f%9.2f", >> "temp.joinfile1b")
# join coords from images 1 and 2
	join("temp.joinfile1a","temp.joinfile1b",out=geomapin)
	del("temp.joinfile*", ver-)

	hsel (refimage, "i_naxis1", "yes") | scan (imax)
	hsel (refimage, "i_naxis2", "yes") | scan (imay)
        print("Next:")
 print("immatch.geomap "//geomapin//" "//pprefix//".geomapout."//ssuffix//" 1 "//imax//" 1 "//imay)

}
else {
        print("WARNING: task `ctio.filecalc' is not loaded!")
        print("Please load it and restart this task.")
        }

if(echog)beep

end

