# ispifirst.cl -- add header params to raw ISPI images and optionally do
#   nonlinearity correction, saturated pixel replacement, & dark subtraction
#
# 1. do nonlinearity correction & add NLCOR (& poss. SATVAL) header card(s)
# 2. hedit image headers, adding: (basically this is task 'hadd.cl', except #0)
# 	1. BPM,BPMASK = bad pixel mask name
#      	2. TIME-OBS from UT
#	3. COADDS -> CO_ADDS; new COADDS & DIVISOR from old COADDS
#      	4. divide by DIVISOR if not already done; add header keyword DIVIDED
#      	5. EXPTIME -> ITIME; 
#	   new EXPTIME  = (ITIME: integration per coadd) * (COADDS: # of coadds)
# 3. subtract dark & add DARKSUB header card
# **: Most arrays give you the counts and integration time of a single coadd.
#    (i.e. all coadds have been averaged internally)  ISPI gives you
#    the TOTAL counts of all coadds but the integration time of a SINGLE coadd.
#    Thus the need to divide by DIVISOR as well as insert EXPTIME as usual.
#
# 030206: adapted for ISPI from irtffirst.cl: 
#	gain & RN free parameters;
#	original header param COADDS moved to CO_ADDS & then ignored;
#	new header param COADDS = DIVISOR created as free parameter;
# 030206: can't dark-subtract in place; suffix="" && darksub=yes won't darksub
#
# **cannot replace saturated pixels except at same time as nonlin. correction

procedure ispifirst(inlist,nonlin,darksub)

string inlist  {prompt="List of images to process (do not use an `@')"}
bool   nonlin=no  {prompt="Do nonlinearity correction?"}
bool   darksub=no {prompt="Subtract off dark frame?"}

string suffix  {prompt="String to be appended to output images",mode="q"}
real   slope   {min=1.,prompt="Slope coefficient of nonlinearity correction",mode="q"}
string intname {prompt="Integration time for nonlinearity correction",mode="q"}
bool   replace=no {prompt="Replace `saturated' pixels?",mode="q"}
real   maxval  {40000.,min=30000.,prompt="Maximum good raw data value",mode="q"}
string dark    {prompt="Name of the dark frame to subtract",mode="q"}
string badmask {prompt="Name of bad pixel mask to add to image header"}
int	coadds	{1,prompt="Number of Coadds"}
real	gain	{4.35,prompt="Gain"}
real	readnoise	{16.,prompt="Read Noise"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *list

begin

string ddark, ssuffix, bbadmask, check, base, ilist
string olist, flat, flatbase, meanfile, currentflat, tempflat, name
string editlist, current, obsdate, rra, ddec, eepoch, oobservatory, minsec
string obstime, timeobs, dateobs, century
real   aairval, ttime, mean, meanbad, x, ganas, ruido
int    ncoadd, ddy, hhr, hhour, mmonth, dday, uttime, mas
bool   ddarksub, ccoords, uut, nnonlin
string  iintname        # equals intname
string  instring        # temporary string
real    sslope          # equals slope of nonlinearity correction
real    mmaxval         # equals maxval
real    valmax          # equals maxval
string  img             # fscanned name of individual input image
string  oimg            # fscanned name of individual output image
real    intime          # equals int_time
real    delaytime       # delay time between reset & first read
real    intfactor       # factor for use in nonlinearity correction
bool    rreplace        # equals replace
bool    go              # just do it
bool    goon            # just do it
struct  theadline       # temporary struct for header information

ilist = inlist
editlist = ilist	# editlist lists images currently being worked on
olist = ilist		# olist lists output images
nnonlin = nonlin
ddarksub = darksub
ssuffix = suffix
if(nnonlin){
# If suffix=="", insert temporary suffix to avoid overwriting
	if(ssuffix==""){
		ssuffix="filthpig"
		beep
		print("WARNING! Input files will be overwritten!")
	}
	olist = ilist//ssuffix  # olist lists output images
        sslope = slope
        iintname = intname
        rreplace = replace
        if(rreplace){
		mmaxval = maxval
		valmax = mmaxval
	}
}
if(ddarksub){
	if(!nnonlin){
		ssuffix = suffix
		olist = ilist//ssuffix  # olist lists output images
	}
	ddark = dark
}
bbadmask = badmask
mas = coadds
ganas = gain
ruido = readnoise

# The "go" parameter won't let the task run if the needed tasks aren't loaded.
go=yes
if(!deftask("fileroot")) {go=no}
if(go) {

# Do nonlinearity correction if desired
goon=no
if(nnonlin){
	print("===> Running nonlinearity correction procedure...")
	list = ilist
	editlist = olist
	while (fscan(list,img) != EOF) {
        	fileroot(img,validim+)
        	oimg=fileroot.root//ssuffix//"."//fileroot.extension
# add output image name to output image list unless suffix=""
        	print(oimg, >> olist)
# do the correction only if input image has not already been corrected
		imgets(img,"NLCOR",>&"dev$null")
		if(imgets.value=="0")goon=yes
# if output image exists...
		if(access(oimg)){
			imgets(oimg,"NLCOR",>&"dev$null")
# ...and has not been nlcorrected, delete it 
			if(imgets.value=="0"){
				imdel(oimg,ver-)
			}
# ...if it has been nlcorrected, print error message & do not do the correction
			else{
				goon=no
       print("Nonlinearity correction already done on "//oimg//"-- not redone.")
			}
		}
		if(goon){
        		imgets(img,iintname)
        		intime=real(imgets.value)
# Find delay time...
			if(intime>=1.381){
        			delaytime=1.
#        			intfactor=1.+(1./intime)
			}
			else{
				delaytime=intime-0.381
#				intfactor=2.-(0.381/intime)
			}
			intfactor=1.+((2.*delaytime)/intime)
# Do nonlinearity correction.  The correction is approximate, but should be
# good for pixels requiring <=10% correction, above which they saturate anyway.
			imcalc(img,oimg,"(im1)/(1.-im1*("//intfactor//"/"//sslope//"))", pixtype="real",nullval=0.,ver-)
			hedit(oimg,"NLCOR",sslope,add+,ver-,show+,update+)
# Replace "saturated" pixels if desired
        		if(rreplace){
                	    mmaxval=(sslope*valmax)/(sslope-valmax*intfactor)
                	    imreplace(oimg,mmaxval,lower=mmaxval,upper=INDEF)
			    hedit(oimg,"SATVAL",mmaxval,add+,ver-,show+,update+)
			}
        	}
# If correction already done on input image, do not redo, just copy to output
		else{
	print("Nonlinearity correction already done on "//img//"-- not redone.")
			if(!access(oimg))imcopy(img,oimg,ver-)
		}
	}
}

# Add the BPM to the image headers
print("===> Adding various parameters to the image headers...")
hedit("@"//editlist,"BPM",bbadmask,add+,ver-,show-,update+)
hedit("@"//editlist,"BPMASK",bbadmask,add+,ver-,show-,update+)

# Add the GAIN and RN to the image headers
hedit("@"//editlist,"GAIN",ganas,add+,ver-,show-,update+)
hedit("@"//editlist,"RN",ruido,add+,ver-,show-,update+)

# Add other goodies to the image headers, just like in the hadd.cl task
list = editlist
while (fscan (list, current) !=EOF) {
  imgets(current,"ut")
  hedit(current,"time-obs",(imgets.value),add+,ver-,show+,update+)
# co_adds from coadds
  imgets(current,"coadds")
  hedit(current,"co_adds",(imgets.value),add+,ver-,show+,update+)
  hedit(current,"coadds",mas,add+,ver-,show+,update+)
  hedit(current,"divisor",mas,add+,ver-,show+,update+)
# divide by DIVISOR
  imgets(current,"divided",>&"dev$null")
  if(imgets.value=="0"){
    imgets(current,"divisor")
    if(imgets.value!="1"){
      imar(current,"/",real(imgets.value),current,noact-,ver-)
      hedit(current,"divided","yes",add+,ver-,show+,update+)
    } else {
      hedit(current,"divided","yes",add+,ver-,show+,update+)
    }
    print("Image "//current//" divided by factor "//imgets.value//" given in DIVISOR.")
  } else {
    print("WARNING! Image "//current//" already divided by DIVISOR!")
  }
# exptime = itime * coadds
  imgets(current,"exptime")
  intime=real(imgets.value)
  hedit(current,"itime",(intime),add+,ver-,show+,update+)
  imgets(current,"coadds")
  ncoadd=real(imgets.value)
  hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
}

# Subtract off Dark Current 
if (ddarksub) {
# If final output list is not same as current list, need to create output list
	if(olist!=editlist){
## if olist doesn't exist, assume darksub'd imgs don't either; create it & dsubs
	    if(!access(olist)) {
	    	print("===> Subtracting off Dark Current...")
		list=editlist
		while (fscan(list,img) != EOF) {
        		fileroot(img,validim+)
        		oimg=fileroot.root//ssuffix
        	    if(fileroot.extension!="")oimg=oimg//"."//fileroot.extension
        		print(oimg, >> olist)
  			imgets(img,"DARKSUB",>&"dev$null")
			if (imgets.value=="0") {
		    		imarith(img, "-", ddark, oimg, verbose-)
		                hedit(oimg,"DARKSUB",ddark,add+,ver-,show+,upd+)
			} else {
  print("DARKSUB keyword exists; ispifirst already run? "//name//" just copied")
		    		imcopy(img, oimg, verbose-)
			}
	    	}
	    }
	} else {
	    print("Darksubbed image list "//olist//" already exists; images assumed to exist too.")
	}
	print(" ") 
        print("===> Dark-subtracted images are listed in the file "//olist//".")
	editlist=olist
}

# If suffix=="", 
if(ssuffix=="filthpig"){
  imdel("@"//ilist,ver-)
  imren("@"//olist,"@"//ilist,ver-)
  del(olist,ver-)
  olist=ilist
}

print(" ") 
print("===> Processed images are listed in the file "//olist//".")
if(nnonlin){
      print("===> You should rerun the task with nonlin=no to properly update")
      print("===> the original image headers, if you plan to use those images.")
}

}

else {
	print("WARNING: tasks fileroot and/or stsdas.toolbox.imgtools.imcalc")
	print("are not loaded!  Please load them and restart this task.")
        }

if(echog)beep

end

