procedure iterstat(image)

# Script to find image statistics excluding deviant pixels

# originally created 4 August 1992 by John Ward
# 930818: revisions by MD
# 950201: changed output formatting-- still prob. w/short values/image names.
# 950209: simple string check for INDEF, for IRAF 2.10.4? patch
# 950314: adjusted use of "sections" task
# 970227: optionally add header keywords ITERMEAN, ITERSIG, ITERMED, ITERMODE
# 970401: optionally add header keyword VARIANCE = ITERSIG**2.
# 990322: added oneparam option; prints INDEF for possibly inaccurate parameters

string	image	{prompt="Input image(s)"}
real	nsigrej	{5.,min=0.,prompt="Number of sigmas for limits"}
int	maxiter	{10,min=1,prompt="Maximum number of iterations"}
bool	print	{yes,prompt="Print final results?"}
bool	verbose	{no,prompt="Show results of all iterations?"}
bool  addheader	{yes,prompt="Put returned values into image header keywords?"}
real	lower	{INDEF,prompt="Initial lower limit for data range"}
real	upper	{INDEF,prompt="Initial upper limit for data range"}
real	mean	{prompt="Returned value of mean"}
real	sigma	{prompt="Returned value of sigma"}
real	variance {prompt="Returned value of variance (sigma**2)"}
real	median	{prompt="Returned value of median"}
real	valmode	{prompt="Returned value of mode"}
#Above must be "valmode" to avoid conflict w/ omnipresent task parameter "mode" 
string oneparam {"all",enum="all|mean|stddev|npix|midpt|mode|rms|median", prompt="Find iterative value of this one parameter only"}
struct	*inimglist

begin

	string	im		# equals image
	string	imglist		# equals image
	string	infile		# temporary list for files
	string  img		# image name from fscan
	string  check		# string for `@filename' checking
	string  blankstring	# long blank string 
	string  headstring	# strings for output formatting
	string  mnstring	# 
	string  sigstring	# 
	string  npxstring	# 
	string  medstring	# 
	string  one_param	#
	real	mn		# mean from imstat
	real	sig		# stddev from imstat
	real	med		# midpt from imstat
	real	mod		# mode from imstat
	real	ll		# lower limit for imstat
	real	ul		# upper limit for imstat
	int	nx, npx		# number of pixels used
	int	m		# dummy for countdown
	int 	i		# length of image string
	int 	j		# dummy integer
	int 	x		# dummy integer

# Expand file lists into temporary files.
imglist = image
infile = imglist
check = substr(imglist,1,1)
if(check=="@") {
	infile=substr(imglist,2,strlen(imglist))
} else {
	infile = mktemp("tmp$iterstat")
	sections (imglist,option="fullname",>infile)
}
inimglist = infile
one_param = oneparam
	if (one_param=="rms") one_param="stddev"
	if (one_param=="median") one_param="midpt"

j=0; med=0.; mod=0.
# Loop through images
while (fscan(inimglist,img) != EOF) {

# Calculate stats using initial lower & upper limits
#	   imstat(img,fields="mean,stddev,npix,midpt,mode",lower=lower,upper=upper,for-) | scan(mn,sig,npx,med,mod)
	if (one_param=="all") { 
imstat(img,fields="mean,stddev,npix,midpt,mode",lower=lower,upper=upper,for-) | scan(mn,sig,npx,med,mod) 
}
	if (one_param=="mean" || one_param=="stddev" || one_param=="npix") imstat(img,fields="mean,stddev,npix",lower=lower,upper=upper,for-) | scan(mn,sig,npx)
	if (one_param=="midpt") imstat(img,fields="mean,stddev,npix,midpt",lower=lower,upper=upper,for-) | scan(mn,sig,npx,med)
	if (one_param=="mode") imstat(img,fields="mean,stddev,npix,mode",lower=lower,upper=upper,for-) | scan(mn,sig,npx,mod)

# If verbose+, print output header
	if(verbose){
	   blankstring="                                                      "
	   i=strlen(str(img))
	   if (i<=6) i=7
           headstring=substr(blankstring,1,i-6)
	   i=strlen(str(mn))
	   if (i<=4) i=5
	   mnstring=substr(blankstring,1,i-4)
	   i=strlen(str(sig))
	   if (i<=3) i=4
	   sigstring=substr(blankstring,1,i-3)
	   i=strlen(str(npx))
	   if (i<=4) i=5
	   npxstring=substr(blankstring,1,i-4)
	   i=strlen(str(med))
	   if (i<=6) i=7
	   medstring=substr(blankstring,1,i-6)
	   print("#IMAGE"//headstring//" MEAN"//mnstring//" RMS"//sigstring//" NPIX"//npxstring//" MEDIAN"//medstring//" MODE")
	}

	m = 1
	while (m <= maxiter)  {
	        if (verbose) print(img," ",mn,sig,npx,med,mod)
# Calculate new lower & upper limits by tossing out values nsigrej from the mean
	   	ll = mn - (nsigrej*sig)
	   	ul = mn + (nsigrej*sig)
		if (lower != INDEF && ll < lower) ll = lower 
		if (upper != INDEF && ul > upper) ul = upper
#	   	imstat(img,fields="mean,stddev,npix,midpt,mode",lower=ll,upper=ul,for-) | scan(mn,sig,nx,med,mod)
	   	if (one_param=="all") { 
imstat(img,fields="mean,stddev,npix,midpt,mode",lower=ll,upper=ul,for-) | scan(mn,sig,nx,med,mod) 
}
	   	if (one_param=="mean"||one_param=="stddev"||one_param=="npix") imstat(img,fields="mean,stddev,npix",lower=ll,upper=ul,for-) | scan(mn,sig,nx)
	   	if (one_param=="midpt") imstat(img,fields="mean,stddev,npix,midpt",lower=ll,upper=ul,for-) | scan(mn,sig,nx,med)
	   	if (one_param=="mode") imstat(img,fields="mean,stddev,npix,mode",lower=ll,upper=ul,for-) | scan(mn,sig,nx,mod)

# If mean is INDEF, set it to zero
 		if(mn==INDEF) {
			mn=0.
print("#WARNING: mean of "//img//" is INDEF!  Setting mean equal to zero.")
			}

# If stddev is INDEF, set it to 32767
 		if(sig==INDEF) {
			sig=32767.
print("#WARNING: stddev of "//img//" is INDEF!  Setting stddev equal to 32767.")
			}
# If midpt (median) is INDEF, set it to the mean 
		if(med==INDEF && (one_param=="all" || one_param=="midpt")) {
			med=mn
print("#WARNING: median of "//img//" is INDEF!  Setting median equal to mean.")
		}
# If mode is INDEF, set it to the mean 
		if(mod==INDEF && (one_param=="all" || one_param=="mode")) {
			mod=mn
print("#WARNING: mode of "//img//" is INDEF!  Setting mode equal to mean.")
		}

# If after this recalculation the same number of pixels are accepted, end loop
	   	if (nx == npx)
	   		break
	   	npx = nx
	   	m = m + 1
	}

# If median (midpt) or mode output was not requested, set to INDEF 
	if (med!=INDEF && (one_param!="all" && one_param!="midpt")) med=INDEF
	if (mod!=INDEF && (one_param!="all" && one_param!="mode")) mod=INDEF

# If first image, print output header
	j+=1
	if(j==1 && !verbose){
	   blankstring="                                                      "
	   i=strlen(str(img))
	   if (i<=6) i=7
           headstring=substr(blankstring,1,i-6)
	   i=strlen(str(mn))
	   if (i<=4) i=5
	   mnstring=substr(blankstring,1,i-4)
	   i=strlen(str(sig))
	   if (i<=3) i=4
	   sigstring=substr(blankstring,1,i-3)
	   i=strlen(str(npx))
	   if (i<=4) i=5
	   npxstring=substr(blankstring,1,i-4)
	   i=strlen(str(med))
	   if (i<=6) i=7
	   medstring=substr(blankstring,1,i-6)
	   if (print && !verbose) print("#IMAGE"//headstring//" MEAN"//mnstring//" RMS"//sigstring//" NPIX"//npxstring//" MEDIAN"//medstring//" MODE")
	}

# Print statistics
	if (print && !verbose) print(img," ",mn,sig,npx,med,mod)
	mean = mn
	sigma = sig
	variance = sig**2.
	median = med
	valmode = mod

	if(addheader) {
		hedit (img, "ITERMEAN", mean, add+, ver-, show-, update+)
		hedit (img, "ITERSIG", sigma, add+, ver-, show-, update+)
		hedit (img, "VARIANCE", variance, add+, ver-, show-, update+)
		hedit (img, "ITERMED", median, add+, ver-, show-, update+)
		hedit (img, "ITERMODE", valmode, add+, ver-, show-, update+)
	}

}

if(check!="@") delete (infile,ver-, >& "dev$null")
inimglist = ""

#!time

end

