# masklet -- break coadded-image object mask into individual-image masklets
#
# 950208: created from dimsum.maskdereg
# 950216: fixed bug in output mask naming
# 960326: allow editing of imshifts file
# 960405: removed angle parameter and added flip option
# 990217: added editfile option
# 000603: added pre-deletion of _objmask

procedure masklet(image,objmask,suffix)

string	image	     {prompt="Image mosaic produced by ircoadd"}
string 	objmask {prompt="Object mask for image mosaic (default=image//objmask)"}
string  suffix="om"  {prompt="Suffix for output object masks"}

string  shiftfile="" {prompt="Image+Shifts file (default=image//.imshifts)"}
bool    editfile=yes {prompt="Edit Image+Shifts file?"}
bool 	flip=no      {prompt="Flip object mask L-R before creating masklets?"}
bool	update=yes   {prompt="Update image headers with object mask name?"}
bool    echog=yes    {prompt="Beep when task is done?"}
struct	*imglist

begin

	string	oimg		# equals image
	string	maskimg		# equals objmask
	string	ssuffix		# equals suffix
	string 	img		# fscanned name of individual input image
	string	word		# original object mask title string
        int     ix,iy   # start pixel coords for input image from .imshifts file
	int 	xe, ye		# end pixel coordinages for input images
	string	ushiftlist	# File outimg//.imshifts produced by shiftfix
	int 	oxdim,oydim 	# dimensions of mask image
	int 	ixdim,iydim 	# dimensions of individual input frames
	string	msect	   # subsection of mask to extract for individual images
	bool	edit_file	# equals editfile

# Get query parameters
	fileroot (image,validim+)
	oimg = fileroot.root
	maskimg = objmask
	if(maskimg=="")maskimg=oimg//"objmask"
	ssuffix = suffix
# Verify that the oimg.imshifts file exists.
	if (shiftfile!="") {ushiftlist=shiftfile}
	  else {ushiftlist = oimg//".imshifts"}
	if (!access(ushiftlist)) {
	   beep
	   print ("ERROR: ircoadd output file ",ushiftlist," does not exist.")
	   return
	}
	edit_file=editfile

# Allow editing of imshifts file
	if (edit_file) {
    print("Edit the imshifts file and replace the flatfielded,")
    print("skysubtracted image names with the image names that will be")
    print("the input to `irflat objmask+' or `irsky objmask+' so that the")
    print("object mask keyword is inserted into the correct image headers.")
    print("#Edit this imshifts file and replace the flatfielded,", >>ushiftlist)
    print("#skysubtracted image names with the image names that will be", >> ushiftlist)
    print("#the input to `irflat objmask+' or `irsky objmask+' so that the", >> ushiftlist)
    print("#object mask keyword is inserted into the correct image headers.", >> ushiftlist)
    vi(ushiftlist)
	}

# Flip mask if desired
	imdel("_objmask",ver-,>&"dev$null")
	if(flip){
		imcopy(maskimg//"[-*,*]","_objmask",ver-)
        } else {
		imcopy(maskimg,"_objmask",ver-)
        }
      hedit("_objmask","title","Object mask for image ",add+,ver-,show-,update+)

# Get dimensions of mask image from image mosaic produced by shiftfix.
	imgets("_objmask","i_naxis1")
	oxdim=int(imgets.value)
	imgets("_objmask","i_naxis2")
	oydim=int(imgets.value)

# Rewind file list.
	imglist=ushiftlist

# Make working mask images, one for each input image, by cutting out 
# appropriate subsection of the large input mask image.   

while (fscan(imglist,img,ix,iy) != EOF) {

	fileroot (img,validim+)
	img = fileroot.root
	print ("Making mask image "//img//ssuffix//".pl for ",img)
	imgets(img,"i_naxis1")
	ixdim=int(imgets.value)
	imgets(img,"i_naxis2")
	iydim=int(imgets.value)
	ix+=1
	iy+=1
	xe=ix + ixdim - 1
	ye=iy + iydim - 1
	msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
	print(msect)
	imcopy("_objmask"//msect,img//ssuffix//".pl",ver-)
	imgets(img//ssuffix//".pl","title")
	word=imgets.value//" "//img
	hedit(img//ssuffix//".pl","title",word,add+,ver-,show-,update+)
# If update=yes, insert mask name into input image headers with OBJMASK card.
	if (update) hedit(img,"OBJMASK",img//ssuffix//".pl",add+,ver-,up+,sho-)

	}

# Clean up.
   print("All done. Next task to run is `irflat objmask+' or `irsky objmask+'.")
	imdel("_objmask",ver-)
	imglist = ""

if(echog)beep

end

