# mkflat.cl -- make domeflats
# 
# calls tasks fileroot, maskstat
# 
# 950329: created
# 971026: minor bug re: "ext" fixed
# 990611: checks for preexisting images & doesn't recreate them
# 000521: changed "rej" param in imcombine to "reject"

procedure mkflat(onlist,offlist,outname,titlestring)

string onlist      {prompt="List of lamp-on flats (do not use an `@')"}
string offlist     {prompt="List of lamp-off flats (do not use an `@')"}
string outname 	   {prompt="Name of output flat image (best include extension)"}
string titlestring {prompt = "Title for output flat image"}

string logfile     {prompt = "Name of log file"}
string badmask     {prompt = "Bad pixel mask to use with `maskstat'"}
real maskval       {prompt = "Value for good pixels in bad pixel mask"}
real gain          {prompt = "Gain of array"}
real noise         {prompt = "Readnoise of array"}
bool   echog        {yes,prompt="Beep when task is done?"}

struct *list

begin

string ddark, ssuffix, check, base, oonlist, oofflist, llogfile
string oout, tstring, off, on, bbadmask, ooutname, ext
real	ggain, readnoise  	# array parameters
real	normval 		# flatfield normalization value
int 	mmaskval 		# = maskval
bool    go              	# just do it
struct  theadline       	# temporary struct for header information

oonlist = onlist
oofflist = offlist
ooutname = outname		# ooutname is output image name
tstring = titlestring

llogfile = logfile
bbadmask = badmask
mmaskval = maskval
ggain = gain
readnoise = noise

# The "go" parameter won't let the task run if the needed tasks aren't loaded.
go=yes
if(!deftask("fileroot")) {go=no}
if(!deftask("maskstat")) {go=no}
if(go) {

	fileroot(ooutname,validim+)	
	oout = fileroot.root		# oout is root of output image name
	ext = fileroot.extension        # ext is extension of output image name
	if(ext!="")ext="."//ext
	off=oout//"off"//ext
	on=oout//"on"//ext

	if (!imaccess(on)) {
		print("===> Creating lamp-on image "//on//"...")
		imcombine ("@"//oonlist, on, combine="average", reject="ccdclip", masktyp="none", lsig=3., hsig=3., scale="mode", zero="none", weight="none", gain=ggain, rdnoise=readnoise, mclip+, logfile="STDOUT", >> llogfile)
	} else {
		print("===> lamp-on image "//on//" already exists!")
	}
	if (!imaccess(off)) {
		print("===> Creating lamp-off image "//off//"...")
		imcombine ("@"//oofflist, off, combine="average", reject="ccdclip", masktyp="none", lsig=3., hsig=3., scale="mode", zero="none", weight="none", gain=ggain, rdnoise=readnoise, mclip+, logfile="STDOUT", >> llogfile)
	} else {
		print("===> lamp-off image "//off//" already exists!")
	}

print("===> Subtracting to create image "//ooutname//"...")
	imar (on, "-", off, ooutname)
	iterstat.nsigrej=5.
	iterstat.maxiter=10
	maskstat(ooutname,bbadmask,mmaskval,iterstat+,ver-,printit+, >>llogfile)
	normval=iterstat.mean
	imar (ooutname, "/", normval, ooutname)
	hedit (ooutname, "NORMVAL", normval, add+, ver-, show-, update+)
	hedit (ooutname, "title", tstring, add+, ver-, show-, update+)

print(" ") 
print("===> Domeflat "//ooutname//" created as "//on//"-"//off//",")
print("===> then normalized by "//normval//".")

}

else {
        print("WARNING: fileroot and/or maskstat tasks are not loaded!")
        print("Please load them and restart this task.")
        }

if(echog)beep

end

