# multimasklet -- remove spurious objects from mask & masklet w/old or new mask
#
# 990512: created from masklet.cl

# redefine multimasklet = phiirs$workdir/multimasklet.cl 

#display image
#display rawmask (object mask)
#display longaxis images (exploded view)
#imexam rawmask	
#after each bad pixel grouping, hit q and enter good-image letter, imexam again?
#	e.g. log_imexam --> log_imexam_A
#take each good image letter and make a separate objmask for each
#	get x,y for a bad pixel w/in area of a spurious object in the mask
#	imreplace x,y to good
#	get coords (listpix) of all bad pixels in x-1:x+1,y-1:y+1
#		if none, end
#		else imreplace to good & repeat with xmin-1:xmax+1,ymin-1:ymax+1
#			where xmin:xmax,ymin:ymax spans the range of bad pixels
#take list of images from each good letter, extract them from *.imshifts,
#	add the mask name to each line, and dump to *.imshiftsmasks

procedure multimasklet (image,rawmask,cleanmask,suffix)

string	image	     {prompt="Coadded Image"}
string rawmask {prompt="Object mask for Coadded Image (default=image//objmask)"}
string cleanmask {prompt="Output cleaned object mask (default=rawmask//clean)"}
string  suffix="om"  {prompt="Suffix for output individual-image object masks"}

string shiftfile="" {prompt="Image+Shifts file (default=image//.imshifts)"}
string shiftmaskfile="" {prompt="Output Image+Shifts+Mask file (default=shiftfile//masks)"}
bool    editfile=yes {prompt="Edit Image+Shifts+Mask file?"}
bool 	flip=no      {prompt="Flip object mask L-R before creating masklets?"}
bool	update=yes   {prompt="Update image headers with object mask name?"}
bool    echog=yes    {prompt="Beep when task is done?"}
struct	*imglist

begin

	string	oimg		# equals image
	string	maskimg		# equals rawmask
	string	clean_mask	# equals cleanmask
	string	ssuffix		# equals suffix
	string 	img		# fscanned name of individual input image
	string	word		# original object mask title string
        int     ix,iy   # start pixel coords for input image from .imshifts file
	int 	xe, ye		# end pixel coordinages for input images
	string	ushiftlist	# equals shiftfile
	string	ushiftmasklist	# equals shiftmaskfile
	int 	oxdim,oydim 	# dimensions of mask image
	int 	ixdim,iydim 	# dimensions of individual input frames
	string	msect	   # subsection of mask to extract for individual images
	bool	edit_file	# equals editfile

# Get query parameters
	fileroot (image,validim+)
	oimg = fileroot.root
	maskimg = rawmask
		if(maskimg=="")maskimg=oimg//"rawmask"
	clean_mask = cleanmask
		if(cleanmask=="")cleanmask=maskimg//"clean"
	ssuffix = suffix

# Verify that the oimg.imshifts file exists.
	if (shiftfile!="") {ushiftlist=shiftfile}
	  else {ushiftlist = oimg//".imshifts"}
	if (!access(ushiftlist)) {
	   beep
	   print ("ERROR: file ",ushiftlist," does not exist.")
	   return
	}
	if (shiftmaskfile!="") {ushiftmasklist=shiftmaskfile}
	  else {ushiftmasklist = ushiftlist//"masks"}
	edit_file=editfile

# ** start editing here

# Allow editing of imshifts file
	if (edit_file) {
    print("Edit the imshifts file and replace the flatfielded,")
    print("skysubtracted image names with the image names that will be")
    print("the input to `irflat objmask+' or `irsky objmask+' so that the")
    print("object mask keyword is inserted into the correct image headers.")
    print("#Edit this imshifts file and replace the flatfielded,", >>ushiftlist)
    print("#skysubtracted image names with the image names that will be", >> ushiftlist)
    print("#the input to `irflat objmask+' or `irsky objmask+' so that the", >> ushiftlist)
    print("#object mask keyword is inserted into the correct image headers.", >> ushiftlist)
    vi(ushiftlist)
	}

# Flip mask if desired
	if(flip){
		imcopy(maskimg//"[-*,*]","_objmask",ver-)
        } else {
		imcopy(maskimg,"_objmask",ver-)
        }
      hedit("_objmask","title","Object mask for image ",add+,ver-,show-,update+)

# Get dimensions of mask image from image mosaic produced by shiftfix.
	imgets("_objmask","i_naxis1")
	oxdim=int(imgets.value)
	imgets("_objmask","i_naxis2")
	oydim=int(imgets.value)

# Rewind file list.
	imglist=ushiftlist

# Make working mask images, one for each input image, by cutting out 
# appropriate subsection of the large input mask image.   

while (fscan(imglist,img,ix,iy) != EOF) {

	fileroot (img,validim+)
	img = fileroot.root
	print ("Making mask image "//img//ssuffix//".pl for ",img)
	imgets(img,"i_naxis1")
	ixdim=int(imgets.value)
	imgets(img,"i_naxis2")
	iydim=int(imgets.value)
	ix+=1
	iy+=1
	xe=ix + ixdim - 1
	ye=iy + iydim - 1
	msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
	print(msect)
	imcopy("_objmask"//msect,img//ssuffix//".pl",ver-)
	imgets(img//ssuffix//".pl","title")
	word=imgets.value//" "//img
	hedit(img//ssuffix//".pl","title",word,add+,ver-,show-,update+)
# If update=yes, insert mask name into input image headers with OBJMASK card.
	if (update) hedit(img,"OBJMASK",img//ssuffix//".pl",add+,ver-,up+,sho-)

	}

# Clean up.
   print("All done. Next task to run is `irflat objmask+' or `irsky objmask+'.")
	imdel("_objmask",ver-)
	imglist = ""

if(echog)beep

end

