# nshift -- find offsets for N images or image sets overlapping a central image
#
# Task calls following other tasks: fileroot, filecalc.
#
# Output:
# prefix.refcoords.suffix	registration objects' reference image coords
# prefix.allcoords.suffix	registration objects' coordinates in all images
# prefix.offsets.suffix		final shifts logfile, + coarse and final shifts
# prefix.imshifts.suffix	image names and rounded int. shifts on same line
# prefix.shiftfrac.suffix	fractional part of final shifts
# prefix.shiftint.suffix	integer part of final shifts
# prefix.roundoff.suffix	final shifts rounded off to integer shifts
# (optional):
# @flatlist//suffix.imh		fractionally-shifted images
# flatlist//suffix		list of fractionally-shifted images
# 
# CHANGES:
# 990422: created from irshift.cl
# 990423: added xregister option (xcor)
# 990519: added zero-out of bad pixels before xcor
#
# --should interpolate over bad pixels before shifting
# --might use iteravg to automatically reject extended objects in refcoords

procedure nshift(filelist,flatlist,refimage,prefix,suffix)

string filelist="" {prompt="File of image lists to find offsets for (no @)"}
string flatlist	   {prompt="Overall list of images to find offsets for (no @)"}
string refimage	      {prompt="Name of (flattened) reference image"}
string prefix	      {prompt="Prefix string for output files"}
string suffix	      {prompt="Suffix string for output files"}

bool fracshift=no     {prompt="Create fractional-pixel-offset images?"}
string interp         {"poly5",enum="nearest|linear|poly3|poly5|spline3",prompt="Interpolation type"}
string boundary       {"nearest",enum="constant|nearest|reflect|wrap",prompt="Boundary type"}
string badmask=""     {prompt="Bad pixel mask for shifted images (blank=old mask)",mode="q"}
bool redo=no          {prompt="Erase files and redo shift-finding if already done?"}
bool useimages=yes    {prompt="Measure shifts from images instead of a file?"}
bool displayim=yes    {prompt="Display images for imexam?"}
bool ptv=no           {prompt="Use phat.ptv instead of display in imexam?"}
string imsection=""   {prompt="Image section to display in imexam?"}
bool editfiles=yes    {prompt="Display coords files for editing?"}
bool xcor=yes         {prompt="Use image cross-correlation to find shifts?"}
int xsize=0 {prompt="X-axis image size (0 for autosense) for cross-correlation"}
int ysize=0 {prompt="Y-axis image size (0 for autosense) for cross-correlation"}
string inputlist      {prompt="File with coordinate info", mode="q"}
bool combdata=no {prompt="Combine shifts with earlier dataset? (NOT TESTED)"}
real xshiftold	      {prompt="Reference image x shift in earlier dataset"}
real yshiftold	      {prompt="Reference image y shift in earlier dataset"}
real radius=5.	      {prompt="Imexamine aperture photometry radius"}
real buffer=10.	      {prompt="Imexamine sky buffer"}
real width=10.	      {prompt="Imexamine sky annulus width"}
real bs=9.	      {prompt="Centering box size for final shift calculation"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *iinputlist
struct *list

begin

string rrefimage, fflatlist, rrefcoords, aallcoords, bbadmask, rroundname, ext
string sshiftlist, coarsefile, ooffname, ssuffix, iintname, ffracname, ooptname
string pprefix, logoffname, sl, sx, img, iinterp, bboundary, im_section
string file_list, nooffname, nlogoffname, naallcoords, nrrefcoords, ncoarsefile
string niintname, nffracname, nrroundname, flat_list, flat_image, tmplist
string tmpfile1, tmpfile2, image_name, msect, tempx, tempy, zero_out
string fr_image_name, fr_rrefimage
real p, pixscale, stcoordx, stcoordy, nrows, ncols, delx, dely, adelx, adely
real xold, yold, bbs, xxshiftold, yyshiftold, xshiftnew, yshiftnew, astcoordx
real astcoordy, nl, nx, x, y, xoff, yoff
real new_x, new_y, old_x, old_y, exact_x, exact_y, step, step_x, step_y
real nxxshiftold, nyyshiftold
int xref, yref, int_x, int_y, n, x_shift, y_shift, xe, ye, ix, iy
int oxdim, oydim, ixdim, iydim
bool uuseimages, ffracshift, ccombdata, go, ddisplayim, edit_files, notalist
bool do_xcor, re_do

# The "go" parameter won't let the task run if you haven't loaded the ctio pkg.
go=yes
if(!deftask("filecalc")) {go=no}
if(!deftask("fileroot")) {go=no}
if(ptv && !deftask("ptv")) {go=no}
if(go) {

file_list = filelist
	notalist = no
flat_list = flatlist
	if (file_list=="") {
		notalist = yes
	}
rrefimage = refimage
pprefix = prefix
ssuffix = suffix

ffracshift = fracshift
	if(ffracshift)bbadmask=badmask
re_do = redo
uuseimages = useimages
ddisplayim = displayim
im_section = imsection
edit_files = editfiles
do_xcor = xcor
ccombdata = combdata
bbs = bs

ooffname = pprefix//".shiftexact."//ssuffix
logoffname = pprefix//".offsets."//ssuffix
rrefcoords = pprefix//".refcoords."//ssuffix
aallcoords = pprefix//".allcoords."//ssuffix
coarsefile = pprefix//".coarse."//ssuffix
iintname = pprefix//".shiftint."//ssuffix
ffracname = pprefix//".shiftfrac."//ssuffix
rroundname = pprefix//".roundoff."//ssuffix

# nshift: Find offsets for each individual list of images in file_list
n = 0
if (notalist) {
	tmplist = flat_list
} else {
	tmplist = file_list
}
list = tmplist
while (fscan (list, fflatlist) !=EOF) {
n = n + 1
nrrefcoords = rrefcoords//"_"//n
naallcoords = aallcoords//"_"//n
ncoarsefile = coarsefile//"_"//n
print ("# "//fflatlist, >> nrrefcoords); print ("# ", >> nrrefcoords)
print ("# "//fflatlist, >> naallcoords); print ("# ", >> naallcoords)

if(ccombdata){
 print(" ")
 print("===> So you wanna add some new images to an old coadd, eh?  Well, first make")
 print("===> sure you're using the same reference image as that old set.  Then, take")
 print("===> a look at the *.offsets.* file from that set, find the fractional")
 print("===> offsets made on the reference image, and enter them here:")
 nxxshiftold=xshiftold
 nyyshiftold=yshiftold
}

# nshift: get reference image coordinates of one object for coarse registration
if(ddisplayim){
	if (!notalist) {
		fields (fflatlist, "1", lines="1-1") | scan (flat_image)
print(" ")
print("===> Displaying "//flat_image//" in frame 2 for convenience,")
	} else {
		fields (flat_list, "1", lines=n//"-"//n) | scan (flat_image)
print(" ")
print("===> Displaying "//flat_image//" in frame 2,")
	}
print("===> and reference image "//rrefimage//" in 1 for examination.")
print(" ")
print("===> Get the coords (hit 'a') in the reference image (frame 1) of ONE object")
print("===> visible in all images, for coarse registration.  Hit 'q' when done.")
print(" ")
	rimexam.radius=radius
	rimexam.buffer=buffer
	rimexam.width=width
	if (ptv) {
		ptv (flat_image, frame=2)
		imexam(rrefimage, frame=1, logfile=nrrefcoords, keep+, display="ptv(image='$1',frame=$2)")
	} else {
		displ (flat_image, frame=2)
		imexam(rrefimage, frame=1, logfile=nrrefcoords, keep+)
	}
}

# First method: ID objects from images
if(uuseimages){
# ... interactively ...
 if(ddisplayim){
  if (ptv) {
	if (notalist) {
		print(" ")
print("===> Get the coordinates (hit 'a') in this image of the object")
print("===> just marked in the reference image.  Hit 'q' when done.")
		print(" ")
		imexam(fflatlist, frame=1, logfile=naallcoords, keep+, display="ptv(image='$1"//im_section//"',frame=$2)")
	} else {
		print(" ")
print("===> Choose one object that appears in all images, & find the position")
print("===> (hit 'a') of this object in each frame (hit 'n' to go on to the")
print("===> next frame, and 'q' when done).  The file of these positions")
print("===> will be displayed for editing when done, in case of mistakes.")
		print(" ")
		imexam("@"//fflatlist, frame=1, logfile=naallcoords, keep+, display="ptv(image='$1"//im_section//"',frame=$2)")
  	}
  } else {
	if (notalist) {
		print(" ")
print("===> Get the coordinates (hit 'a') in this image of the object")
print("===> just marked in the reference image.  Hit 'q' when done.")
		print(" ")
		imexam(fflatlist, frame=1, logfile=naallcoords, keep+, display="display(image='$1"//im_section//"',frame=$2)")
	} else {
		print(" ")
print("===> Choose one object that appears in all images, & find the position")
print("===> (hit 'a') of this object in each frame (hit 'n' to go on to the")
print("===> next frame, and 'q' when done).  The file of these positions")
print("===> will be displayed for editing when done, in case of mistakes.")
		print(" ")
		imexam("@"//fflatlist, frame=1, logfile=naallcoords, keep+, display="display(image='$1"//im_section//"',frame=$2)")
	}
  }
 } else {
# ... or non-interactively ...
  print("===> Images not displayed.")
 }

 if(edit_files){
  print(" ")
  print("===> In case of mistakes, both coordinate files can now be edited.")
  vi(nrrefcoords)
  vi(naallcoords)
 }

 print(" ")
 print("===> Finding the coarse offsets...")
 fields (nrrefcoords, "1-2") | scan (xref, yref)
 if((!access(ncoarsefile)) || (re_do && access(ncoarsefile))) {
	del(ncoarsefile,ver-,>&"dev$null")
 	lintran(naallcoords,x1=0.001,y1=0.001,x2=xref,y2=yref,angle=180, >ncoarsefile)
 }
}

# Second method: estimate shifts from telescope offsets
if(!uuseimages){
 iinputlist=inputlist
 print("Generating initial shift file from file "//inputlist//".")
 while (fscan (iinputlist, pixscale, astcoordx, astcoordy, nrows, ncols, adelx, adely) !=EOF) {
	stcoordx=astcoordx/pixscale
	stcoordy=astcoordy/pixscale
	delx=adelx/pixscale
	dely=adely/pixscale
	x=stcoordx
	xold=stcoordx
	y=stcoordy
	yold=stcoordy

#procedure:i=1 (0,-dely); i=2 (-delx,+dely); i=3 (-delx,-dely); i=4(-delx,+dely)
	for (i=1; i<=nrows; i+=1) {
		for (j=1; j<=ncols; j+=1) {
			y=yold+dely*(-1)**(i+1)
			print(x, y, >> naallcoords)
			yold=y
		} 
		x=xold+delx
		xold=x
		y=yold+dely*(-1)**(i+1)
		yold=y
	}
}

 print("===> Edit the file with :wq twice.")
	
 vi("-c ':g/0\.\ /s//0.00 /g'", naallcoords)
 vi("-c ':g/0\.$/s//0.00/g'", naallcoords)

 print(" ")
 print("===> Type in the reference image offsets (printed 2 lines below) when")
 print("===> requested, to zero the coordinate system on the reference image.")

 del("tempsortlist", ver-, >& "dev$null")
 join(fflatlist, naallcoords, out="tempsortlist")
 grep(rrefimage,"tempsortlist")
 del("tempsortlist",ver-)

 print("===> Finding the coarse offsets...")
 if((!access(ncoarsefile)) || (re_do && access(ncoarsefile))) {
	del(ncoarsefile,ver-)
 	lintran(naallcoords, x2=xref, y2=yref, angle=180, > ncoarsefile)
 }
}

# nshift: end of file of lists
}


# Common ending for both approaches.
print(" ")
print("===> Finding the final offsets...")

# nshift: create coarsefile from all ncoarsefile
print(" ")
print("===> Finding exact offsets...")
if ((!access(coarsefile)) || (re_do && access(coarsefile))) {
	del (coarsefile, ver-, >& "dev$null")
	concat (coarsefile//"_?", coarsefile, append-)
}
# nshift: Get reference image coords of all reference objects
# nshift: new xregister method
if (do_xcor) {
	print ("Cross-correlating individual images with central image...")
	ixdim=xsize; iydim=ysize
#print (ixdim, iydim)
        imgets(rrefimage,"i_naxis1")
        oxdim=int(imgets.value)
        imgets(rrefimage,"i_naxis2")
        oydim=int(imgets.value)
#print (oxdim, oydim)
	tmpfile1=mktemp("tmpfile1"); tmpfile2=mktemp("tmpfile2")
	fields (coarsefile, "1-2", >> tmpfile1)
	join (flat_list//","//tmpfile1, >> tmpfile2)
        list=tmpfile2
	if ((!access(pprefix//".xregout."//ssuffix)) || (re_do && access(pprefix//".xregout."//ssuffix))) {
            while (fscan(list,image_name,x_shift,y_shift) !=EOF) {
# zero out bad pixels if not already done
		imgets(image_name,"ZEROOUT")
		zero_out=imgets.value
		if(zero_out=="0"){
			imgets(image_name,"BPMASK")
			if(imgets.value!="0"){
imar(image_name,"*",imgets.value,image_name,pixt="real",calct="real",ver-,noac-)
hedit(image_name,"ZEROOUT",imgets.value,add+,ver-,show-,update+)
			} else {
print("Bad pixels not zeroed for "//image_name//", and no BP mask in header!")
			}
		}
                if(xsize==0){
                        imgets(image_name,"i_naxis1")
                        ixdim=int(imgets.value)
                }
                if(ysize==0){
                        imgets(image_name,"i_naxis2")
                        iydim=int(imgets.value)
                }
#print (x_shift, y_shift)
#print (ixdim, iydim)
                ix=int(x_shift)+1-14; if(ix<1)ix=1
                iy=int(y_shift)+1-14; if(iy<1)iy=1
                xe=int(x_shift) + ixdim + 14; if(xe>oxdim)xe=oxdim
                ye=int(y_shift) + iydim + 14; if(ye>oydim)ye=oydim
#print (ix, iy)
#print (xe, ye)
                msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
   print("Rough shift from "//image_name//" to "//rrefimage//" is "//x_shift//" "//y_shift)
		fileroot (image_name, validim+); fr_image_name = fileroot.root
		fileroot (rrefimage, validim+); fr_rrefimage= fileroot.root
		if (fr_image_name != fr_rrefimage) {
			immatch.xreg (image_name, rrefimage, msect, pprefix//".xregout."//ssuffix, datab+, records="", xlag=-1*x_shift, ylag=-1*y_shift, xwindow=25, ywindow=25, append+)
		} else {
			print("", >> pprefix//".xregout."//ssuffix)
			print("begin   "//image_name, >> pprefix//".xregout."//ssuffix)
			print("        xshift          0.",>>pprefix//".xregout."//ssuffix)
			print("        yshift          0.",>>pprefix//".xregout."//ssuffix)
		} 
	    }
	}
        tempx = mktemp("tempx"); tempy = mktemp("tempy")
        grep ('xshift', pprefix//".xregout."//ssuffix) | fields ("junk", "2", >> tempx)
        grep ('yshift', pprefix//".xregout."//ssuffix) | fields ("junk", "2", >> tempy)
        join(tempx,tempy,output=ooffname)
        del (tempx, ver-); del (tempy, ver-)
        del (tmpfile1, ver-); del (tmpfile2, ver-)

} else {

# nshift: irshift-style method
# nshift: Get reference image coords of all reference objects
	if(ddisplayim){
	print(" ")
print("===> Get the coordinates (type 'a') in the reference image of ALL objects you")
print("     wish to use for registration, in no particular order.  Hit 'q' when done.")
	print(" ")
		rimexam.radius=radius
		rimexam.buffer=buffer
		rimexam.width=width
		if (ptv) {
			imexam(rrefimage, frame=1, logfile=rrefcoords, keep+, display="ptv(image='$1',frame=$2)")
		} else {
			imexam(rrefimage, frame=1, logfile=rrefcoords, keep+)
		}
	}
	if(edit_files){
 		print(" ")
print("===> In case of mistakes, the reference coords file can now be edited.")
 		vi(rrefcoords)
	}

# nshift: set fflatlist to flat_list (all images)
	fflatlist=flat_list

	del(logoffname, ver-, >& "dev$null")
	imcentroid("@"//fflatlist, rrefimage, rrefcoords, shift=coarsefile, boxsize=bbs, > logoffname)
	del("temp.joinfile", ver-, >& "dev$null")
	del("temp.joinfile?", ver-, >& "dev$null")
# read in length of temp.joinfile0 and cut out last two lines
	copy(logoffname,"temp.joinfile0")
	count("temp.joinfile0") | scan (nl)
	nx=nl-2
	head("temp.joinfile0", nlines=nx, >> "temp.joinfile1")
# find line on which the string "X-s" appears and cut out that line & all before
	!grep -n X-s temp.joinfile1 > temp.joinfile2
        type("temp.joinfile2") | scan (sl)
	x=stridx(":",sl)
	sx=substr(sl,1,x)
	x=nx-real(sx)
	tail("temp.joinfile1", nlines=x, >> "temp.joinfile")
# Cut down on number of output files
	print(" ", >> logoffname)
	print("#Coarse offsets:", >> logoffname)
	print(" ", >> logoffname)
	type(coarsefile, >> logoffname)
#	del(coarsefile, ver-)

# Create exact shifts list
	if(ccombdata){
# Extract the new reference image shifts from the offsets file
  		del("temp.joinfile?",ver-, >& "dev$null")
  		grep(rrefimage,"temp.joinfile") | fields ("junk", "2", >> "temp.joinfile3")
  		type("temp.joinfile3") | average >> "temp.joinfile4"
  		xshiftnew=real(average.sum)
  		del("temp.joinfile?",ver-)
  		grep(rrefimage,"temp.joinfile") | fields ("junk", "4", >> "temp.joinfile3")
  		type("temp.joinfile3") | average >> "temp.joinfile4"
  		yshiftnew=real(average.sum)
  		del("temp.joinfile?",ver-)
# Want to take the calculated shift ($2 for x), cancel out the newly calculated
# reference image shift ($2 - xshiftnew = -xnew+$2 = -1*xnew+$2) and add in
# the old reference image shift (xold+(-1*xnew+$2)
  		filecalc("temp.joinfile",xxshiftold//"+(-1*"//xshiftnew//"+$2);"//yyshiftold//"+(-1*"//yshiftnew//"+$4)",format="%8.2f%8.2f  ", >> ooffname)
  		flpr; flpr; flpr
	} else {
  		fields("temp.joinfile", "2,4", >> ooffname)
	}
	del("temp.joinfile*", ver-)
# nshift: end of old irshift method
}

# Create integer shifts list
if (!access(iintname) || (re_do && access(iintname))) {
	list=ooffname
	while(fscan(list,x,y) !=EOF) {
		print(int(x),int(y), >> iintname)
	}
}
# Create fractional shifts list
if (!access(ffracname) || (re_do && access(ffracname))) {
	join(iintname,ooffname,out="temp.joinfile")
	filecalc("temp.joinfile","$3-$1;$4-$2",forma="%6.2f%6.2f", >> ffracname)
	flpr; flpr; flpr
}

# Create optimum rounded-off integer offsets
if (!access(rroundname) || (re_do && access(rroundname))) {
    print(" ")
    print("===> Finding optimum rounded-off integer offsets...")
    old_x = 999.; old_y = 999.
# for each 0.01 pixel step...
    for (i=-50; i<50; i+=1) {
	step = i / 100.
#	print (i)
# ...recalculate integer offsets...
	list=ooffname
	del (rroundname, ver-, >& "dev$null")
	del ("temp."//rroundname, ver-, >& "dev$null")
        while(fscan(list,xoff,yoff) !=EOF) {
		xoff=xoff+step
		yoff=yoff+step
                if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                        print(xoff,yoff, >> "temp."//rroundname)
                } else {
                        if((abs(xoff))<=0.5) {
                                xoff=0.
                        }
                        if((abs(yoff))<=0.5) {
                                yoff=0.
                        }
                        print(xoff,yoff, >> "temp."//rroundname)
                }
     filecalc("temp."//rroundname, "$1;$2", format="%4.0f %4.0f", >> rroundname)
     		flpr; flpr; flpr
                del ("temp."//rroundname, ver-, >& "dev$null")
        }
        del ("temp."//rroundname, ver-, >& "dev$null")
# calculate sum of squares of residuals of exact offsets - new integer offsets
	filecalc (ooffname//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
	flpr; flpr; flpr
	new_x=average.sumsqrs
	filecalc (ooffname//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
	flpr; flpr; flpr
	new_y=average.sumsqrs
	if (new_x < old_x) { 
		old_x = new_x
		step_x = step
	}
	if (new_y < old_y) {
		old_y = new_y
		step_y = step
	}
    }
# recalculate final integer offsets
    list=ooffname
    del (rroundname, ver-)
    del ("temp."//rroundname, ver-, >& "dev$null")
    while(fscan(list,xoff,yoff) !=EOF) {
	xoff=xoff+step_x
	yoff=yoff+step_y
        if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                print(xoff,yoff, >> "temp."//rroundname)
        } else {
                if((abs(xoff))<=0.5) {
                       xoff=0.
                }
                if((abs(yoff))<=0.5) {
                       yoff=0.
                }
                print(xoff,yoff, >> "temp."//rroundname)
        }
        filecalc("temp."//rroundname,"$1;$2",format="%4.0f %4.0f", >>rroundname)
	flpr; flpr; flpr
        del ("temp."//rroundname, ver-, >& "dev$null")
    }
    del ("temp."//rroundname, ver-, >& "dev$null")
}

if (!access(pprefix//".imshifts."//ssuffix) || (re_do && access(pprefix//".imshifts."//ssuffix))) {
    print(" ", >> logoffname)
    print("#Final offsets:", >> logoffname)
    print(" ", >> logoffname)
    type(ooffname, >> logoffname)
    del("temp.joinfile", ver-)
    join(flat_list,rroundname,out=pprefix//".imshifts."//ssuffix)
}

# Produce photometric time series from *allcoords* file
# fields(aallcoords, "7") | graph (title="Flux of centering object as f(image number)", point-)

# Create fractional-pixel-shifted images
if(ffracshift){
	sshiftlist=fflatlist//ssuffix
	print(" ")
	print("===> Creating fractional-pixel-shifted images listed in "//sshiftlist//".")
	list=fflatlist
	while(fscan(list,img) !=EOF) {
		fileroot(img,validim+)
		img=fileroot.root
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		print(img//ssuffix//ext, >> sshiftlist)
	}
	iinterp=interp
	bboundary=boundary
	imshift("@"//fflatlist, "@"//sshiftlist, shifts=ffracname, interp=iinterp, bound=bboundary)
	if(bbadmask!="") {
		hedit("@"//sshiftlist,"BPM",bbadmask,add+,ver-,show-,update+)
		hedit("@"//sshiftlist,"BPMASK",bbadmask,add+,ver-,show-,update+)
	}
}

print("...it is accomplished.  Next you should run `ircoadd' using")
if(ffracshift)print(sshiftlist//" and "//iintname//" or")
print(fflatlist//" and "//rroundname//".")

} else {
   print("WARNING: tasks `filecalc', `fileroot', or package `phat' (if ptv=yes)"
	print("are not loaded!  Please load them and restart this task.")
}

if(echog)beep

end

