# onis3ew -- shift & coadd ONIS CNOC 3-field east-west dither pattern

# 990420 -- created from onis6pt.cl
# 990422 -- nshiftall portion created
# 990423 -- started adding ability to handle non-overlapping images in nshiftall
# 990518 -- make sure indiv. images are multiplied by BPM before xcor
# 990519 -- sigmanorm of long-axis coadds commented out (not used)
# 030522 -- changed shall to shfull for offsets of all individ. imgs

# ** ideally should take offset file input to PROSPERO, and dither file too,
# 	and figure out image pairs for itself

# ** could do xcor in a chain: after coaddlongaxes, 
# 	displ AB #1 CD #2 & measure 1-star offset, then 
# 	EHI #3 FGI #4 MLK #1 ON #2 TSP #3 URQ #4 VWX #1
# ** would allow all shifts to be done at once

# --ideally would include multimasklet.cl instead of masklet.cl,
#	to identify & exclude spurious objects from bright star residuals.

# ** interactively ident star resids?  when?  (& make bpms)
# ** ideally want to ID star resids as you go thru the images in order
#    could do this in chainshift 
#    but how to apply marked x,y to final objmask image?
# ** if you have objmask and objmasknospur, want to tell it which single-letter
#	images take which object mask.  
# ** better write masklet to handle this.  But will it make a diff?

#get x,y for a bad pixel w/in area of a spurious object in the mask
#imreplace x,y to good
#get coords (listpix) of all bad pixels in x-1:x+1,y-1:y+1
#	if none, end
#	if some, imreplace to good and repeat with xmin-1:xmax+1,ymin-1:ymax+1
#		 where the xmin:xmax,ymin:ymax spans the range of bad pixels

# ** in pre-review: display xm=0.5 ym=0.5 with estimated offsets?

# layout of 3-field E-W dither pattern & image pairings (long axis vertical):
# T20 U21 V22	
# S19 R18 W23	TSP 		URQ 		VWX
# P16 Q17 X24	20/19/16        21/18/17        22/23/24
# O15 B2  C3	ON 		BA 		CD
# N14 A1  D4	15/14		2/1		4/5
# M13 F6  E5	MLK 		FGJ 		EHI
# L12 G7  H8	13/12/11	6/7/10		5/8/9
# K11 J10 I9	

procedure onis3ew(inlist,imageroot,niter)

string	inlist		{prompt="List of input images"}
string	imageroot	{prompt="Root name for output files"}
int    	niter=1     	{prompt="Number to append to output files"}

string	pairing="all"	{enum="MLK|FGJ|EHI|ON|BA|CD|TSP|URQ|VWX|all",prompt="Images pairings to process (or all)"}
bool inspect=yes	{prompt="Inspect image lists before proceeding?"}
bool shiftlongaxes=yes	{prompt="Find shifts for image pairings w/overlapping long axes?"}
bool coaddlongaxes=yes	{prompt="Coadd image pairings w/overlapping long axes?"}
bool nshiftall=yes	{prompt="Find shifts between all image pairings?"}
bool prereview=yes	{prompt="Display all image for pre-nshift check?"}
bool p_tv=yes	     {prompt="Use ptv instead of display for pre-nshift check?"}
bool use_images=yes  {prompt="Measure shifts from images (not file) in nshift?"}
bool display_im=yes	{prompt="Display images for imexam in nshift?"}
bool edit_files=yes     {prompt="Edit coords files after creation in nshift?"}
bool exclude	   {prompt="Exclude any image pairings from first-pass nshift?"}
string exstring	{prompt="Enter code(s) for excluded pairing(s) separated by commas (MLK|FGJ|EHI|ON|BA|CD|TSP|URQ|VWX)"}
bool redo=no		{prompt="Redo shift-finding if already done?"}
bool coaddallimages=yes	{prompt="Coadd all images?"}
bool secondpassxcor=yes	{prompt="Cross-correlate all images with coadded image?"}
bool coaddsecondpass=yes {prompt="Coadd images after second-pass cross-correlation?"}
bool domkobjmask=yes	{prompt="Make coadded-image object mask (INTERACTIVE)?"}
bool domasklet=yes	{prompt="Make individual-image object masks?"}
string imextn="fits"	{prompt="Image extension"}
int xsize=512		{prompt="X-axis individual image size (0 for autosense)"}
int ysize=1024		{prompt="Y-axis individual image size (0 for autosense)"}
bool echog=yes		{prompt="Beep when task is done?"}

struct *list
 
begin
 
string in_list, image_root, p_air, ref_image, tmpheaders, sigprefix, rroundname
string theory, method_shortaxes, tempfile2, msect, image_name, tempx, tempy
string rvw_image, ex_string, current_string, future_string, in_string, im_extn
string zero_out
int xmin, ymin, xint, yint, old_xint, old_yint, xmax_bcad, xmin_fead, ix, iy
int xmin_tspurq, xmax_tspurq, xtspurq, ytspurq, tspn1
int xmin_vwxurq, xmax_vwxurq, xvwxurq, yvwxurq, vwxn1
int xmin_mlkfgj, xmax_mlkfgj, xmlkfgj, ymlkfgj, mlkn1
int xmin_ehifgj, xmax_ehifgj, xehifgj, yehifgj, ehin1
int n_iter, xbcad, ybcad, xfead, yfead, bcn1, fen1, x_shift, y_shift, xe, ye
int xref, yref, int_x, int_y, junkint, ilen, ipos
int oxdim, oydim     # dimensions of mask image
int ixdim, iydim     # dimensions of individual input frames
int xbamost, ybamost, xbaba, ybaba, xcdba, ycdba, xehiba, yehiba, xfgjba, yfgjba
int xmlkba, ymlkba, xonba, yonba, xtspba, ytspba, xurqba, yurqba, xvwxba, yvwxba
real baba_xshiftexact, baba_yshiftexact, bamost_xshiftexact, bamost_yshiftexact
real cdba_xshiftexact, cdba_yshiftexact, ehiba_xshiftexact, ehiba_yshiftexact
real fgjba_xshiftexact, fgjba_yshiftexact, mlkba_xshiftexact, mlkba_yshiftexact 
real onba_xshiftexact, onba_yshiftexact, tspba_xshiftexact, tspba_yshiftexact
real urqba_xshiftexact, urqba_yshiftexact, vwxba_xshiftexact, vwxba_yshiftexact 
real xbcad2, ybcad2, xfead2, yfead2, bcad_x, bcad_y, fead_x, fead_y
real xtspurq2, ytspurq2, tspurq_xlag, tspurq_ylag
real xvwxurq2, yvwxurq2, vwxurq_xlag, vwxurq_ylag
real xurq, yurq, xtsp, ytsp, xvwx, yvwx
real xmlkfgj2, ymlkfgj2, mlkfgj_xlag, mlkfgj_ylag
real xehifgj2, yehifgj2, ehifgj_xlag, ehifgj_ylag
real xfgj, yfgj, xmlk, ymlk, xehi, yehi
real xad, yad, xbc, ybc, xfe, yfe, bcad_xlag, bcad_ylag, fead_xlag, fead_ylag
real bcad_xfirstpass, bcad_yfirstpass, fead_xfirstpass, fead_yfirstpass
real tspurq_xfirstpass, tspurq_yfirstpass, vwxurq_xfirstpass, vwxurq_yfirstpass 
real mlkfgj_xfirstpass, mlkfgj_yfirstpass, ehifgj_xfirstpass, ehifgj_yfirstpass 
real xoff, yoff, step, new_x, new_y, old_x, old_y, step_x, step_y, xsig, ysig
bool in_spect, shift_longaxes, echo_g, coadd_longaxes, abort, go, do_masklet
bool coadd_allimages, xcor_secondpass, coadd_secondpass, do_mkobjmask, re_do
bool im_display, images_use, shift_shortaxes, coadd_shortaxes, xreg_secondpass
bool nshift_all, files_edit, ex_clude, pt_v, pre_review

in_list=inlist
image_root=imageroot
n_iter=niter
p_air=pairing

in_spect=inspect
shift_longaxes=shiftlongaxes
images_use=use_images
im_display=display_im
files_edit=edit_files
ex_string=""
re_do=redo
coadd_longaxes=coaddlongaxes
nshift_all=nshiftall
pre_review=prereview
pt_v=p_tv
coadd_allimages=coaddallimages
xcor_secondpass=secondpassxcor
coadd_secondpass=coaddsecondpass
do_mkobjmask=domkobjmask
do_masklet=domasklet
im_extn=imextn
echo_g=echog

rroundname=image_root//".xregshifts.shall"//n_iter
sigprefix=sigmanorm.prefix
baba_xshiftexact = 0; baba_yshiftexact = 0; cdba_xshiftexact = 0
cdba_xshiftexact = 0; ehiba_xshiftexact = 0; ehiba_yshiftexact = 0
fgjba_xshiftexact = 0; fgjba_yshiftexact = 0; mlkba_xshiftexact = 0
mlkba_yshiftexact = 0; onba_xshiftexact = 0; onba_yshiftexact = 0
tspba_xshiftexact = 0; tspba_yshiftexact = 0; urqba_xshiftexact = 0
urqba_yshiftexact = 0; vwxba_xshiftexact = 0; vwxba_yshiftexact = 0

go=yes

if(!defpac("imgtools")) {go=no}
if(go) {


# Separate input list into lists of images overlapping along long axes
print ("\n===> Checking image pairing lists...")
tmpheaders=mktemp("tmpheaders")
hsel ("@"//in_list, "$I,i_title", yes, >> tmpheaders)
if (!access(in_list//"_BA")) {
 grep ('Position\ \ 2\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_BA")
 grep ('Position\ \ 1\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_BA")
} else {
 print (in_list//"_BA already exists; not created.")
}
if (!access(in_list//"_ON")) {
 grep ('Position\ 15\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ON")
 grep ('Position\ 14\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ON")
} else {
 print (in_list//"_ON already exists; not created.")
}
if (!access(in_list//"_CD")) {
 grep ('Position\ \ 3\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_CD")
 grep ('Position\ \ 4\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_CD")
} else {
 print (in_list//"_CD already exists; not created.")
}
if (!access(in_list//"_MLK")) {
 grep ('Position\ 13\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_MLK")
 grep ('Position\ 12\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_MLK")
 grep ('Position\ 11\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_MLK")
 grep ('Position\ 13\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ML")
 grep ('Position\ 12\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_ML")
 grep ('Position\ 11\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_K")
 print (in_list//"_ML", >> "l_"//in_list//"_MLK")
 print (in_list//"_K", >> "l_"//in_list//"_MLK")
} else {
 print (in_list//"_MLK etc. already exist; not created.")
}
if (!access(in_list//"_FGJ")) {
 grep ('Position\ \ 6\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FGJ")
 grep ('Position\ \ 7\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FGJ")
 grep ('Position\ 10\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FGJ")
 grep ('Position\ \ 6\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FG")
 grep ('Position\ \ 7\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FG")
 grep ('Position\ 10\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_J")
 print (in_list//"_FG", >> "l_"//in_list//"_FGJ")
 print (in_list//"_J", >> "l_"//in_list//"_FGJ")
} else {
 print (in_list//"_FGJ etc. already exist; not created.")
}
if (!access(in_list//"_EHI")) {
 grep ('Position\ \ 5\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_EHI")
 grep ('Position\ \ 8\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_EHI")
 grep ('Position\ \ 9\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_EHI")
 grep ('Position\ \ 5\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_EH")
 grep ('Position\ \ 8\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_EH")
 grep ('Position\ \ 9\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_I")
 print (in_list//"_EH", >> "l_"//in_list//"_EHI")
 print (in_list//"_I", >> "l_"//in_list//"_EHI")
} else {
 print (in_list//"_EHI etc. already exist; not created.")
}
if (!access(in_list//"_TSP")) {
 grep ('Position\ 20\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_TSP")
 grep ('Position\ 19\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_TSP")
 grep ('Position\ 16\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_TSP")
 grep ('Position\ 20\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_TS")
 grep ('Position\ 19\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_TS")
 grep ('Position\ 16\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_P")
 print (in_list//"_TS", >> "l_"//in_list//"_TSP")
 print (in_list//"_P", >> "l_"//in_list//"_TSP")
} else {
 print (in_list//"_TSP etc. already exist; not created.")
}
if (!access(in_list//"_URQ")) {
 grep ('Position\ 21\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_URQ")
 grep ('Position\ 18\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_URQ")
 grep ('Position\ 17\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_URQ")
 grep ('Position\ 21\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_UR")
 grep ('Position\ 18\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_UR")
 grep ('Position\ 17\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_Q")
 print (in_list//"_UR", >> "l_"//in_list//"_URQ")
 print (in_list//"_Q", >> "l_"//in_list//"_URQ")
} else {
 print (in_list//"_URQ etc. already exist; not created.")
}
if (!access(in_list//"_VWX")) {
 grep ('Position\ 22\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_VWX")
 grep ('Position\ 23\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_VWX")
 grep ('Position\ 24\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_VWX")
 grep ('Position\ 22\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_VW")
 grep ('Position\ 23\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_VW")
 grep ('Position\ 24\ of\ 24', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_X")
 print (in_list//"_VW", >> "l_"//in_list//"_VWX")
 print (in_list//"_X", >> "l_"//in_list//"_VWX")
} else {
 print (in_list//"_VWX etc. already exist; not created.")
}
del(tmpheaders,ver-)
if (in_spect) vi (in_list//"_?? "//in_list//"_???")


# Do irshift on lists of images overlapping along long axes
if (shift_longaxes) {
print("\n===> Finding shifts for image pairings overlapping along long axes...")
	if (p_air=="BA" || p_air=="all") {
		fields (in_list//"_BA", 1, lines="2-2") | scan (ref_image)
		if (!access(image_root//".roundoff.shBA"//n_iter)) {
			print("\n----> Overlap region is LOWER part of the reference image; using irshift <----")
			irshift (in_list//"_BA", ref_image, image_root, "shBA"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="ON" || p_air=="all") {
		fields (in_list//"_ON", 1, lines="2-2") | scan (ref_image)
		if (!access(image_root//".roundoff.shON"//n_iter)) {
			print("\n----> Overlap region is LOWER part of the reference image; using irshift <----")
			irshift (in_list//"_ON", ref_image, image_root, "shON"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="CD" || p_air=="all") {
		fields (in_list//"_CD", 1, lines="2-2") | scan (ref_image)
		if (!access(image_root//".roundoff.shCD"//n_iter)) {
			print("\n----> Overlap region is LOWER part of the reference image; using irshift <----")
			irshift (in_list//"_CD", ref_image, image_root, "shCD"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="MLK" || p_air=="all") {
		fields (in_list//"_MLK", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shMLK"//n_iter)) {
      			print("\n----> The overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_MLK", in_list//"_MLK", ref_image, image_root, "shMLK"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="FGJ" || p_air=="all") {
		fields (in_list//"_FGJ", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shFGJ"//n_iter)) {
      			print("\n----> The overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_FGJ", in_list//"_FGJ", ref_image, image_root, "shFGJ"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="EHI" || p_air=="all") {
		fields (in_list//"_EHI", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shEHI"//n_iter)) {
      			print("\n----> The overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_EHI", in_list//"_EHI", ref_image, image_root, "shEHI"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="TSP" || p_air=="all") {
		fields (in_list//"_TSP", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shTSP"//n_iter)) {
      			print("\n----> The overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_TSP", in_list//"_TSP", ref_image, image_root, "shTSP"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="URQ" || p_air=="all") {
		fields (in_list//"_URQ", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shURQ"//n_iter)) {
      			print("\n----> The overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_URQ", in_list//"_URQ", ref_image, image_root, "shURQ"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
	if (p_air=="VWX" || p_air=="all") {
		fields (in_list//"_VWX", 1, lines="6-6") | scan (ref_image)
		if (!access(image_root//".roundoff.shVWX"//n_iter)) {
      			print("\n----> The overlap region is the ENTIRE reference image; using nshift <----")
			nshift ("l_"//in_list//"_VWX", in_list//"_VWX", ref_image, image_root, "shVWX"//n_iter, useimages=images_use, displayim=im_display, editfiles=files_edit)
		}
	}
}


# Coadd images overlapping along long axes
if (coadd_longaxes) {
	print ("\n===> Coadding image pairings overlapping along long axes...")
	if (p_air=="BA" || p_air=="all") {
		imdel (image_root//"BA"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"BA"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"BA"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"BA"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_BA", image_root//"BA"//n_iter, "", image_root//".roundoff.shBA"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"BA"//n_iter)
	}
	if (p_air=="ON" || p_air=="all") {
		imdel (image_root//"ON"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"ON"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"ON"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"ON"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_ON", image_root//"ON"//n_iter, "", image_root//".roundoff.shON"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"ON"//n_iter)
	}
	if (p_air=="CD" || p_air=="all") {
		imdel (image_root//"CD"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"CD"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"CD"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"CD"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_CD", image_root//"CD"//n_iter, "", image_root//".roundoff.shCD"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"CD"//n_iter)
	}
	if (p_air=="MLK" || p_air=="all") {
		imdel (image_root//"MLK"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"MLK"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"MLK"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"MLK"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_MLK", image_root//"MLK"//n_iter, "", image_root//".roundoff.shMLK"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"MLK"//n_iter)
	}
	if (p_air=="FGJ" || p_air=="all") {
		imdel (image_root//"FGJ"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"FGJ"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"FGJ"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"FGJ"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_FGJ", image_root//"FGJ"//n_iter, "", image_root//".roundoff.shFGJ"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"FGJ"//n_iter)
	}
	if (p_air=="EHI" || p_air=="all") {
		imdel (image_root//"EHI"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"EHI"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"EHI"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"EHI"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_EHI", image_root//"EHI"//n_iter, "", image_root//".roundoff.shEHI"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"EHI"//n_iter)
	}
	if (p_air=="TSP" || p_air=="all") {
		imdel (image_root//"TSP"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"TSP"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"TSP"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"TSP"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_TSP", image_root//"TSP"//n_iter, "", image_root//".roundoff.shTSP"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"TSP"//n_iter)
	}
	if (p_air=="URQ" || p_air=="all") {
		imdel (image_root//"URQ"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"URQ"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"URQ"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"URQ"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_URQ", image_root//"URQ"//n_iter, "", image_root//".roundoff.shURQ"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"URQ"//n_iter)
	}
	if (p_air=="VWX" || p_air=="all") {
		imdel (image_root//"VWX"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"VWX"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"VWX"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"VWX"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_VWX", image_root//"VWX"//n_iter, "", image_root//".roundoff.shVWX"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_allatonce", makeavg+, flip-)
#		sigmanorm (image_root//"VWX"//n_iter)
	}
}


# Find shifts between all image pairings
if (p_air=="all" && nshift_all) {
    #if (!access(image_root//".shiftexact.shall"//n_iter)) {
# ** should test this out
	if (re_do) {
		del (image_root//".imshifts.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".offsets.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".roundoff.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftexact.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftfrac.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftint.shall*"//n_iter, ver-, >& "dev$null")
		del (image_root//".imshifts.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".offsets.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".roundoff.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftexact.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftfrac.shmost*"//n_iter, ver-, >& "dev$null")
		del (image_root//".shiftint.shmost*"//n_iter, ver-, >& "dev$null")
		print ("\n*allcoords*, *refcoords*, & *coarse* preserved; all else will be recreated.")
	}
	if (!access(in_list//"_coadd_"//n_iter)) {
		print (image_root//"BA"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"CD"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"EHI"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"FGJ"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"MLK"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"ON"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"TSP"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"URQ"//n_iter, >> in_list//"_coadd_"//n_iter)
		print (image_root//"VWX"//n_iter, >> in_list//"_coadd_"//n_iter)
	}
# initial pre-review 
	if (pre_review) {
	  print ("Central image will be displayed in frame 1, and others in 2.")
	  print ("Note any images that do not overlap the central image.")
	  list=in_list//"_coadd_"//n_iter
	  junkint = 1
	  while (fscan (list, rvw_image) !=EOF) {
		if (junkint==1) {
			if (pt_v) {
				ptv (rvw_image, frame=1)
			} else {
				display (rvw_image, frame=1)
			}
		} else {
			if (pt_v) {
				imexam (rvw_image, frame=2, keep-, display="ptv(image='$1',frame=2)")
			} else {
				display (rvw_image, junkint)
				imexam (rvw_image, frame=junkint, keep-, display="display(image='$1',frame=$2)")
				#imexam (rvw_image, frame=2, keep-, display="display(image='$1',frame=2)")
			}
		}
		junkint = junkint + 1
		if (junkint==5) junkint = 2
	  }
	}
# exclude any images from nshift at first?
	print ("")
	ex_clude=exclude
# if excluding any images: 
	if (ex_clude) {
	    if (!access(image_root//".shiftexact.shmost"//n_iter)) {
# **redo here too? not a global redo, since need to get paramaters below for use in coadding
		del ("_coaddTmp?_", ver-, >& "dev$null")
# get code(s) of excluded image(s)
		ex_string=exstring
# create file of codes of images excluded from most-images coadd
#		files (image_root//ex_string//n_iter, >> "_coaddTmp1_")
		files (ex_string, >> "_coaddTmp1_")
# create file of images included in most-images coadd
		copy (in_list//"_coadd_"//n_iter, "_coaddTmp2_", ver-)
		!fgrep -v -f _coaddTmp1_ _coaddTmp2_ > _coaddTmp3_
# **redo/!redo instead of delete & recreate?
		del (in_list//"_coaddmost_"//n_iter, ver-, >& "dev$null")
		rename ("_coaddTmp3_", in_list//"_coaddmost_"//n_iter, field="all", >& "dev$null")
# create file of images included in full coadd
# **redo/!redo instead of delete & recreate?
		del (in_list//"_coaddrest_"//n_iter, ver-, >& "dev$null")
		print (image_root//"BA"//n_iter, > "_coaddTmp4_")
		!fgrep -f _coaddTmp1_ _coaddTmp2_ >> _coaddTmp4_
		print (image_root//"most"//n_iter//"_1stpass", >> "_coaddTmp4_")
		type ("_coaddTmp4_", >> in_list//"_coaddrest_"//n_iter)
# find shifts for most-images coadd
# **redo here too?
		if (!access(image_root//".roundoff.shmost"//n_iter)) nshift ("", in_list//"_coaddmost_"//n_iter, image_root//"BA"//n_iter, image_root, "shmost"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
    		if (!access(image_root//".imshifts.shmost"//n_iter)) join (in_list//"_coaddmost_"//n_iter, image_root//".shiftexact.shmost"//n_iter, >> image_root//".imshifts.shmost"//n_iter)
    		grep ("BA", image_root//".imshifts.shmost"//n_iter) | scan (theory, baba_xshiftexact, baba_yshiftexact)
    		grep ("CD", image_root//".imshifts.shmost"//n_iter) | scan (theory, cdba_xshiftexact, cdba_yshiftexact)
  		grep ("EHI", image_root//".imshifts.shmost"//n_iter) | scan (theory, ehiba_xshiftexact, ehiba_yshiftexact)
  		grep ("FGJ", image_root//".imshifts.shmost"//n_iter) | scan (theory, fgjba_xshiftexact, fgjba_yshiftexact)
  		grep ("MLK", image_root//".imshifts.shmost"//n_iter) | scan (theory, mlkba_xshiftexact, mlkba_yshiftexact)
    		grep ("ON", image_root//".imshifts.shmost"//n_iter) | scan (theory, onba_xshiftexact, onba_yshiftexact)
  		grep ("TSP", image_root//".imshifts.shmost"//n_iter) | scan (theory, tspba_xshiftexact, tspba_yshiftexact)
  		grep ("URQ", image_root//".imshifts.shmost"//n_iter) | scan (theory, urqba_xshiftexact, urqba_yshiftexact)
  		grep ("VWX", image_root//".imshifts.shmost"//n_iter) | scan (theory, vwxba_xshiftexact, vwxba_yshiftexact)
	    }
	} else {
# if not excluding any images: 
		if (!access(image_root//".shiftexact.shall"//n_iter)) nshift ("", in_list//"_coadd_"//n_iter, image_root//"BA"//n_iter, image_root, "shall"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
    		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="1-1")|scan(baba_xshiftexact,baba_yshiftexact)
    		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="2-2")|scan(cdba_xshiftexact,cdba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="3-3")|scan(ehiba_xshiftexact,ehiba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="4-4")|scan(fgjba_xshiftexact,fgjba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="5-5")|scan(mlkba_xshiftexact,mlkba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="6-6")|scan(onba_xshiftexact,onba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="7-7")|scan(tspba_xshiftexact,tspba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="8-8")|scan(urqba_xshiftexact,urqba_yshiftexact)
  		fields(image_root//".shiftexact.shall"//n_iter,"1-2",lines="9-9")|scan(vwxba_xshiftexact,vwxba_yshiftexact)
	}
    #}
    if(baba_xshiftexact>=0)xbaba=int(0.5+baba_xshiftexact)
    if(baba_yshiftexact>=0)ybaba=int(0.5+baba_yshiftexact)
    if(baba_xshiftexact<0)xbaba=int(-0.5+baba_xshiftexact)
    if(baba_yshiftexact<0)ybaba=int(-0.5+baba_yshiftexact)
    if(cdba_xshiftexact>=0)xcdba=int(0.5+cdba_xshiftexact)
    if(cdba_yshiftexact>=0)ycdba=int(0.5+cdba_yshiftexact)
    if(cdba_xshiftexact<0)xcdba=int(-0.5+cdba_xshiftexact)
    if(cdba_yshiftexact<0)ycdba=int(-0.5+cdba_yshiftexact)
    if(ehiba_xshiftexact>=0)xehiba=int(0.5+ehiba_xshiftexact)
    if(ehiba_yshiftexact>=0)yehiba=int(0.5+ehiba_yshiftexact)
    if(ehiba_xshiftexact<0)xehiba=int(-0.5+ehiba_xshiftexact)
    if(ehiba_yshiftexact<0)yehiba=int(-0.5+ehiba_yshiftexact)
    if(fgjba_xshiftexact>=0)xfgjba=int(0.5+fgjba_xshiftexact)
    if(fgjba_yshiftexact>=0)yfgjba=int(0.5+fgjba_yshiftexact)
    if(fgjba_xshiftexact<0)xfgjba=int(-0.5+fgjba_xshiftexact)
    if(fgjba_yshiftexact<0)yfgjba=int(-0.5+fgjba_yshiftexact)
    if(mlkba_xshiftexact>=0)xmlkba=int(0.5+mlkba_xshiftexact)
    if(mlkba_yshiftexact>=0)ymlkba=int(0.5+mlkba_yshiftexact)
    if(mlkba_xshiftexact<0)xmlkba=int(-0.5+mlkba_xshiftexact)
    if(mlkba_yshiftexact<0)ymlkba=int(-0.5+mlkba_yshiftexact)
    if(onba_xshiftexact>=0)xonba=int(0.5+onba_xshiftexact)
    if(onba_yshiftexact>=0)yonba=int(0.5+onba_yshiftexact)
    if(onba_xshiftexact<0)xonba=int(-0.5+onba_xshiftexact)
    if(onba_yshiftexact<0)yonba=int(-0.5+onba_yshiftexact)
    if(tspba_xshiftexact>=0)xtspba=int(0.5+tspba_xshiftexact)
    if(tspba_yshiftexact>=0)ytspba=int(0.5+tspba_yshiftexact)
    if(tspba_xshiftexact<0)xtspba=int(-0.5+tspba_xshiftexact)
    if(tspba_yshiftexact<0)ytspba=int(-0.5+tspba_yshiftexact)
    if(urqba_xshiftexact>=0)xurqba=int(0.5+urqba_xshiftexact)
    if(urqba_yshiftexact>=0)yurqba=int(0.5+urqba_yshiftexact)
    if(urqba_xshiftexact<0)xurqba=int(-0.5+urqba_xshiftexact)
    if(urqba_yshiftexact<0)yurqba=int(-0.5+urqba_yshiftexact)
    if(vwxba_xshiftexact>=0)xvwxba=int(0.5+vwxba_xshiftexact)
    if(vwxba_yshiftexact>=0)yvwxba=int(0.5+vwxba_yshiftexact)
    if(vwxba_xshiftexact<0)xvwxba=int(-0.5+vwxba_xshiftexact)
    if(vwxba_yshiftexact<0)yvwxba=int(-0.5+vwxba_yshiftexact)
}


# coadd all images at once (w/intermediate most-image coadd if needed)
if (p_air=="all" && coadd_allimages) {

# first set minimum x,y offsets to be 0 & 0 (independently)
	if (!access(image_root//".roundoff.shBA"//n_iter//"min0")) {
		list=image_root//".roundoff.shBA"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shBA"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shBA"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shON"//n_iter//"min0")) {
		list=image_root//".roundoff.shON"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shON"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shON"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shCD"//n_iter//"min0")) {
		list=image_root//".roundoff.shCD"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shCD"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shCD"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shEHI"//n_iter//"min0")) {
		list=image_root//".roundoff.shEHI"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shEHI"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shEHI"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shFGJ"//n_iter//"min0")) {
		list=image_root//".roundoff.shFGJ"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shFGJ"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shFGJ"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shMLK"//n_iter//"min0")) {
		list=image_root//".roundoff.shMLK"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shMLK"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shMLK"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shTSP"//n_iter//"min0")) {
		list=image_root//".roundoff.shTSP"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shTSP"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shTSP"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shURQ"//n_iter//"min0")) {
		list=image_root//".roundoff.shURQ"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shURQ"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shURQ"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shVWX"//n_iter//"min0")) {
		list=image_root//".roundoff.shVWX"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shVWX"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shVWX"//n_iter//"min0")
	}

# if not excluding any images:
	if (!exclude) {
		if(ex_string=="")ex_string=exstring

# now offset by offsets to the central image (BA)
		if(!access(in_list//"_all")) concat (in_list//"_BA,"//in_list//"_CD,"//in_list//"_EHI,"//in_list//"_FGJ,"//in_list//"_MLK,"//in_list//"_ON,"//in_list//"_TSP,"//in_list//"_URQ,"//in_list//"_VWX", in_list//"_all")
#		if(!access(image_root//".roundoff.shall"//n_iter)) {
		if(!access(image_root//".roundoff.shfull"//n_iter)) {
			filecalc (image_root//".roundoff.shBA"//n_iter//"min0", "$1+"//xbaba//";$2+"//ybaba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shCD"//n_iter//"min0", "$1+"//xcdba//";$2+"//ycdba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shEHI"//n_iter//"min0", "$1+"//xehiba//";$2+"//yehiba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shFGJ"//n_iter//"min0", "$1+"//xfgjba//";$2+"//yfgjba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shMLK"//n_iter//"min0", "$1+"//xmlkba//";$2+"//ymlkba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shON"//n_iter//"min0", "$1+"//xonba//";$2+"//yonba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shTSP"//n_iter//"min0", "$1+"//xtspba//";$2+"//ytspba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shURQ"//n_iter//"min0", "$1+"//xurqba//";$2+"//yurqba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
			filecalc (image_root//".roundoff.shVWX"//n_iter//"min0", "$1+"//xvwxba//";$2+"//yvwxba, format="%4.0f %4.0f", >> image_root//".roundoff.shfull"//n_iter)
		}
# do the coadd 
		print ("\n===> Coadding all images at once...")
		ircoadd (in_list//"_all", image_root//"all"//n_iter//"_1stpass", "", image_root//".roundoff.shfull"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")

# if excluding any images: 
	} else {

		if(!access(in_list//"_most")) {
# create list of all image lists 
			print (in_list//"_BA\n"//in_list//"_CD\n"//in_list//"_EHI\n"//in_list//"_FGJ\n"//in_list//"_MLK\n"//in_list//"_ON\n"//in_list//"_TSP\n"//in_list//"_URQ\n"//in_list//"_VWX", > "_coaddTmp5_")
# create list of all image lists to be included in coadd
			!fgrep -v -f _coaddTmp1_ _coaddTmp5_ > _coaddTmp6_
			concat ("@_coaddTmp6_", in_list//"_most")
		}
# offset the shift lists of all the images to be included in the coadd
# by their offsets to the central image (BA)
		if(!access(image_root//".1stpass.shmost"//n_iter)) {
# create list of codes of all image lists 
			print ("BA\nCD\nEHI\nFGJ\nMLK\nON\nTSP\nURQ\nVWX", > "_coaddTmp7_")
# create list of codes of all image lists to be included in coadd
			!fgrep -v -f _coaddTmp1_ _coaddTmp7_ > _coaddTmp8_
			list = "_coaddTmp8_"
			while (fscan (list, in_string) !=EOF) {
				if (in_string=="BA") {
					filecalc (image_root//".roundoff.shBA"//n_iter//"min0", "$1+"//xbaba//";$2+"//ybaba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="CD") {
					filecalc (image_root//".roundoff.shCD"//n_iter//"min0", "$1+"//xcdba//";$2+"//ycdba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="EHI") {
					filecalc (image_root//".roundoff.shEHI"//n_iter//"min0", "$1+"//xehiba//";$2+"//yehiba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="FGJ") {
					filecalc (image_root//".roundoff.shFGJ"//n_iter//"min0", "$1+"//xfgjba//";$2+"//yfgjba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="MLK") {
					filecalc (image_root//".roundoff.shMLK"//n_iter//"min0", "$1+"//xmlkba//";$2+"//ymlkba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="ON") {
					filecalc (image_root//".roundoff.shON"//n_iter//"min0", "$1+"//xonba//";$2+"//yonba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="TSP") {
					filecalc (image_root//".roundoff.shTSP"//n_iter//"min0", "$1+"//xtspba//";$2+"//ytspba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="URQ") {
					filecalc (image_root//".roundoff.shURQ"//n_iter//"min0", "$1+"//xurqba//";$2+"//yurqba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				} else if (in_string=="VWX") {
					filecalc (image_root//".roundoff.shVWX"//n_iter//"min0", "$1+"//xvwxba//";$2+"//yvwxba, format="%4.0f %4.0f", >> image_root//".1stpass.shmost"//n_iter)
				}
			}
			del ("_coaddTmp?_", ver-, >& "dev$null")
		}

# do the most-images coadd 
		if (!access(image_root//"most"//n_iter//"_1stpass."//im_extn)) {
			print ("\n===> Coadding most images at once...")
			ircoadd (in_list//"_most", image_root//"most"//n_iter//"_1stpass", "", image_root//".1stpass.shmost"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
		}

# do nshift on most-images coadd
		if (!access(image_root//".roundoff.shrest"//n_iter)) nshift ("", in_list//"_coaddrest_"//n_iter, image_root//"most"//n_iter//"_1stpass", image_root, "shrest"//n_iter, redo=re_do, useimages=images_use, displayim=im_display, editfiles=files_edit)
# get shifts for images excluded from most-images coadd
		if (!access(image_root//".imshifts.shrest"//n_iter)) join(in_list//"_coaddrest_"//n_iter,image_root//".shiftexact.shrest"//n_iter, >>image_root//".imshifts.shrest"//n_iter)
		grep("BA",image_root//".imshifts.shrest"//n_iter)|scan(theory, bamost_xshiftexact, bamost_yshiftexact)
    		if(bamost_xshiftexact>=0)xbamost=int(0.5+bamost_xshiftexact)
    		if(bamost_yshiftexact>=0)ybamost=int(0.5+bamost_yshiftexact)
    		if(bamost_xshiftexact<0)xbamost=int(-0.5+bamost_xshiftexact)
    		if(bamost_yshiftexact<0)ybamost=int(-0.5+bamost_yshiftexact)
                if (stridx(ex_string,"BA")!=0) {
			grep("BA",image_root//".imshifts.shrest"//n_iter)|scan(theory, baba_xshiftexact, baba_yshiftexact)
    			if(baba_xshiftexact>=0)xbaba=int(0.5+baba_xshiftexact)-xbamost
    			if(baba_yshiftexact>=0)ybaba=int(0.5+baba_yshiftexact)-ybamost
    			if(baba_xshiftexact<0)xbaba=int(-0.5+baba_xshiftexact)-xbamost
    			if(baba_yshiftexact<0)ybaba=int(-0.5+baba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"CD")!=0) {
			grep("CD",image_root//".imshifts.shrest"//n_iter)|scan(theory,cdba_xshiftexact,cdba_yshiftexact)
    			if(cdba_xshiftexact>=0)xcdba=int(0.5+cdba_xshiftexact)-xbamost
    			if(cdba_yshiftexact>=0)ycdba=int(0.5+cdba_yshiftexact)-ybamost
    			if(cdba_xshiftexact<0)xcdba=int(-0.5+cdba_xshiftexact)-xbamost
    			if(cdba_yshiftexact<0)ycdba=int(-0.5+cdba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"EHI")!=0) {
			grep("EHI",image_root//".imshifts.shrest"//n_iter)|scan(theory,ehiba_xshiftexact,ehiba_yshiftexact)
    			if(ehiba_xshiftexact>=0)xehiba=int(0.5+ehiba_xshiftexact)-xbamost
    			if(ehiba_yshiftexact>=0)yehiba=int(0.5+ehiba_yshiftexact)-ybamost
    			if(ehiba_xshiftexact<0)xehiba=int(-0.5+ehiba_xshiftexact)-xbamost
    			if(ehiba_yshiftexact<0)yehiba=int(-0.5+ehiba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"FGJ")!=0) {
			grep("FGJ",image_root//".imshifts.shrest"//n_iter)|scan(theory,fgjba_xshiftexact,fgjba_yshiftexact)
    			if(fgjba_xshiftexact>=0)xfgjba=int(0.5+fgjba_xshiftexact)-xbamost
    			if(fgjba_yshiftexact>=0)yfgjba=int(0.5+fgjba_yshiftexact)-ybamost
    			if(fgjba_xshiftexact<0)xfgjba=int(-0.5+fgjba_xshiftexact)-xbamost
    			if(fgjba_yshiftexact<0)yfgjba=int(-0.5+fgjba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"MLK")!=0) {
			grep("MLK",image_root//".imshifts.shrest"//n_iter)|scan(theory,mlkba_xshiftexact,mlkba_yshiftexact)
    			if(mlkba_xshiftexact>=0)xmlkba=int(0.5+mlkba_xshiftexact)-xbamost
    			if(mlkba_yshiftexact>=0)ymlkba=int(0.5+mlkba_yshiftexact)-ybamost
    			if(mlkba_xshiftexact<0)xmlkba=int(-0.5+mlkba_xshiftexact)-xbamost
    			if(mlkba_yshiftexact<0)ymlkba=int(-0.5+mlkba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"ON")!=0) {
			grep("ON",image_root//".imshifts.shrest"//n_iter)|scan(theory,onba_xshiftexact,onba_yshiftexact)
    			if(onba_xshiftexact>=0)xonba=int(0.5+onba_xshiftexact)-xbamost
    			if(onba_yshiftexact>=0)yonba=int(0.5+onba_yshiftexact)-ybamost
    			if(onba_xshiftexact<0)xonba=int(-0.5+onba_xshiftexact)-xbamost
    			if(onba_yshiftexact<0)yonba=int(-0.5+onba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"TSP")!=0) {
			grep("TSP",image_root//".imshifts.shrest"//n_iter)|scan(theory,tspba_xshiftexact,tspba_yshiftexact)
    			if(tspba_xshiftexact>=0)xtspba=int(0.5+tspba_xshiftexact)-xbamost
    			if(tspba_yshiftexact>=0)ytspba=int(0.5+tspba_yshiftexact)-ybamost
    			if(tspba_xshiftexact<0)xtspba=int(-0.5+tspba_xshiftexact)-xbamost
    			if(tspba_yshiftexact<0)ytspba=int(-0.5+tspba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"URQ")!=0) {
			grep("URQ",image_root//".imshifts.shrest"//n_iter)|scan(theory,urqba_xshiftexact,urqba_yshiftexact)
    			if(urqba_xshiftexact>=0)xurqba=int(0.5+urqba_xshiftexact)-xbamost
    			if(urqba_yshiftexact>=0)yurqba=int(0.5+urqba_yshiftexact)-ybamost
    			if(urqba_xshiftexact<0)xurqba=int(-0.5+urqba_xshiftexact)-xbamost
    			if(urqba_yshiftexact<0)yurqba=int(-0.5+urqba_yshiftexact)-ybamost
                }
		if (stridx(ex_string,"VWX")!=0) {
			grep("VWX",image_root//".imshifts.shrest"//n_iter)|scan(theory,vwxba_xshiftexact,vwxba_yshiftexact)
    			if(vwxba_xshiftexact>=0)xvwxba=int(0.5+vwxba_xshiftexact)-xbamost
    			if(vwxba_yshiftexact>=0)yvwxba=int(0.5+vwxba_yshiftexact)-ybamost
    			if(vwxba_xshiftexact<0)xvwxba=int(-0.5+vwxba_xshiftexact)-xbamost
    			if(vwxba_yshiftexact<0)yvwxba=int(-0.5+vwxba_yshiftexact)-ybamost
		}

# now offset by offsets to the central image (BA)
		if(!access(in_list//"_all")) concat (in_list//"_BA,"//in_list//"_CD,"//in_list//"_EHI,"//in_list//"_FGJ,"//in_list//"_MLK,"//in_list//"_ON,"//in_list//"_TSP,"//in_list//"_URQ,"//in_list//"_VWX", in_list//"_all")
#		if(!access(image_root//".1stpass.shall"//n_iter)) {
		if(!access(image_root//".1stpass.shfull"//n_iter)) {
			filecalc (image_root//".roundoff.shBA"//n_iter//"min0", "$1+"//xbaba//";$2+"//ybaba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shCD"//n_iter//"min0", "$1+"//xcdba//";$2+"//ycdba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shEHI"//n_iter//"min0", "$1+"//xehiba//";$2+"//yehiba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shFGJ"//n_iter//"min0", "$1+"//xfgjba//";$2+"//yfgjba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shMLK"//n_iter//"min0", "$1+"//xmlkba//";$2+"//ymlkba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shON"//n_iter//"min0", "$1+"//xonba//";$2+"//yonba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shTSP"//n_iter//"min0", "$1+"//xtspba//";$2+"//ytspba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shURQ"//n_iter//"min0", "$1+"//xurqba//";$2+"//yurqba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
			filecalc (image_root//".roundoff.shVWX"//n_iter//"min0", "$1+"//xvwxba//";$2+"//yvwxba, format="%4.0f %4.0f", >> image_root//".1stpass.shfull"//n_iter)
		}

# do the full coadd 
		print ("\n===> Coadding all images at once...")
		ircoadd (in_list//"_all", image_root//"all"//n_iter//"_1stpass", "", image_root//".1stpass.shfull"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
	}
}


# cross-correlate individual images with coadded image
if (p_air=="all" && xcor_secondpass) {
	ixdim=xsize; iydim=ysize
	if(!access(sigprefix//image_root//"all"//n_iter//"_1stpass."//im_extn)){
		print ("\n===> Creating constant-RMS coadded image...")
		sigmanorm(image_root//"all"//n_iter//"_1stpass."//im_extn)
	}
	imgets(sigprefix//image_root//"all"//n_iter//"_1stpass","i_naxis1")
	oxdim=int(imgets.value)
	imgets(sigprefix//image_root//"all"//n_iter//"_1stpass","i_naxis2")
	oydim=int(imgets.value)
	list=image_root//"all"//n_iter//"_1stpass.imshifts"
	if (!access(image_root//"all"//n_iter//"_xregout")) {
	  print("\n===> Cross-correlating individual images w/coadded image...")
		while (fscan(list,image_name,x_shift,y_shift) !=EOF) {
# zero out bad pixels if not already done
                        imgets(image_name,"ZEROOUT")
                        zero_out=imgets.value
                        if(zero_out=="0"){
                                imgets(image_name,"BPMASK")
                                if(imgets.value!="0"){
imar(image_name,"*",imgets.value,image_name,pixt="real",calct="real",ver-,noac-)
hedit(image_name,"ZEROOUT",imgets.value,add+,ver-,show-,update+)
                                } else {
print("Bad pixels not zeroed for "//image_name//", and no BP mask in header!")
				}
                        }
			if(xsize==0){
				imgets(image_name,"i_naxis1")
				ixdim=int(imgets.value)
			}
			if(ysize==0){
				imgets(image_name,"i_naxis2")
				iydim=int(imgets.value)
			}
			ix=x_shift+1-14; if(ix<1)ix=1
                	iy=y_shift+1-14; if(iy<1)iy=1
                	xe=x_shift + ixdim + 14; if(xe>oxdim)xe=oxdim
                	ye=y_shift + iydim + 14; if(ye>oydim)ye=oydim
        		msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
			print ("Rough shift from "//image_name//" to "//sigprefix//image_root//"all"//n_iter//"_1stpass is "//x_shift//" "//y_shift//" pixels")
			immatch.xreg (image_name, sigprefix//image_root//"all"//n_iter//"_1stpass", msect, image_root//"all"//n_iter//"_xregout", datab+, records="", xlag=-1*x_shift, ylag=-1*y_shift, xwindow=25, ywindow=25, append+)
		}
	}
	tempx = mktemp("tempx"); tempy = mktemp("tempy")
	grep ('xshift', image_root//"all"//n_iter//"_xregout") | fields ("junk", "2", >> tempx)
	grep ('yshift', image_root//"all"//n_iter//"_xregout") | fields ("junk", "2", >> tempy)
#	if(!access(image_root//".xregexact.shall"//n_iter)) join (tempx, tempy, output=image_root//".xregexact.shall"//n_iter)
	if(!access(image_root//".xregexact.shfull"//n_iter)) join (tempx, tempy, output=image_root//".xregexact.shfull"//n_iter)
	del (tempx, ver-); del (tempy, ver-)
# Create optimum rounded-off integer offsets
	if (!access(rroundname)) {
      print("\n===> Finding optimum second-pass rounded-off integer offsets...")
	    old_x = 999.; old_y = 999.
# for each 0.01 pixel step...
	    for (i=-50; i<50; i+=1) {
		step = i / 100.
#		print (i)
# ...recalculate integer offsets...
#		list=image_root//".xregexact.shall"//n_iter
		list=image_root//".xregexact.shfull"//n_iter
		del (rroundname, ver-, >& "dev$null")
        	while(fscan(list,xoff,yoff) !=EOF) {
			xoff=xoff+step
			yoff=yoff+step
                	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                        	print(xoff,yoff, >> "temp."//rroundname)
                	} else {
                        	if((abs(xoff))<=0.5) {
                                	xoff=0.
                        	}
                        	if((abs(yoff))<=0.5) {
                                	yoff=0.
                        	}
                        	print(xoff,yoff, >> "temp."//rroundname)
                	}
     filecalc("temp."//rroundname, "$1;$2", format="%4.0f %4.0f", >> rroundname)
     			flpr; flpr; flpr
                	del ("temp."//rroundname, ver-, >& "dev$null")
        	}
        	del ("temp."//rroundname, ver-, >& "dev$null")
# calculate sum of squares of residuals of exact offsets - new integer offsets
#		filecalc (image_root//".xregexact.shall"//n_iter//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
		filecalc (image_root//".xregexact.shfull"//n_iter//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_x=average.sumsqrs
#		filecalc (image_root//".xregexact.shall"//n_iter//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
		filecalc (image_root//".xregexact.shfull"//n_iter//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_y=average.sumsqrs
		if (new_x < old_x) { 
			old_x = new_x
			step_x = step
		}
		if (new_y < old_y) {
			old_y = new_y
			step_y = step
		}
	    }
# recalculate final integer offsets
#	    list=image_root//".xregexact.shall"//n_iter
	    list=image_root//".xregexact.shfull"//n_iter
	    del(rroundname,ver-)
	    del ("temp."//rroundname, ver-, >& "dev$null")
	    while(fscan(list,xoff,yoff) !=EOF) {
		xoff=xoff+step_x
		yoff=yoff+step_y
        	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                	print(xoff,yoff, >> "temp."//rroundname)
        	} else {
                	if((abs(xoff))<=0.5) {
                       		xoff=0.
                	}
                	if((abs(yoff))<=0.5) {
                       		yoff=0.
                	}
                	print(xoff,yoff, >> "temp."//rroundname)
        	}
         filecalc("temp."//rroundname,"$1;$2",format="%4.0f %4.0f",>>rroundname)
		flpr; flpr; flpr
        	del ("temp."//rroundname, ver-, >& "dev$null")
	    }
	del ("temp."//rroundname, ver-, >& "dev$null")
	}
	del ("temp."//rroundname, ver-, >& "dev$null")
#	filecalc (rroundname//","//image_root//".1stpass.shall"//n_iter, "$1@1-$1@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shfull"//n_iter, "$1@1-$1@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shfull"//n_iter, "$1@1-$1@2-"//average.mean, lines="1-", >> "temp."//rroundname) 
	filecalc (rroundname//","//image_root//".1stpass.shfull"//n_iter, "$2@1-$2@2", lines="1-") | average
	filecalc (rroundname//","//image_root//".1stpass.shfull"//n_iter, "$2@1-$2@2-"//average.mean, lines="1-", >> "temp."//rroundname) 
	graph ("temp."//rroundname, point-, title="mean-subtracted 2ndpass - 1stpass x and y offsets")
	del ("temp."//rroundname, ver-, >& "dev$null")
}


# Coadd images after second-pass cross-correlation
if (p_air=="all" && coadd_secondpass) {
	print ("\n===> Coadding after second-pass cross-correlation...")
	ircoadd(in_list//"_all", image_root//"all"//n_iter, "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg-, flip-, imcoaddpars="phiirs$ircoaddonis.par")
#	ircoadd(in_list//"_all", image_root//"all"//n_iter//"avg", "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_all", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
}


# Make coadded-image object mask
if (p_air=="all" && do_mkobjmask) {
	print ("\n===> Making coadded-image object mask...")
	mkobjmask (image_root//"all"//n_iter//"."//im_extn, "", suffix="objmask", nsmooth=3, subsample=3, filtsize=15, ngrow=1, interact+)
}


# Make individual-image object masks
if (p_air=="all" && do_masklet) {
	print ("\n===> Making individual-image object masks...")
	masklet (image_root//"all"//n_iter, "", "om", shiftfile="", editfile+, flip-, update+)
}


} else {
	print ("\nWARNING! Package stsdas.imgtools not loaded... task aborted.")
}

if (echo_g) beep

end

