# onis6pt -- shift & coadd ONIS CNOC 6-point roundabout dither pattern

# 990413 -- created
# 990419 -- major modifications
# 990521 -- added some helpful screen output during shortaxes xreg step;
#		skip shiftlongaxes step if redo- & *.roundoff.* exists

# layout of 6-point roundabout dither pattern (long axis vertical):
# CDE
# BAF

procedure onis6pt(inlist,imageroot,niter)

string	inlist		{prompt="List of input images"}
string	imageroot	{prompt="Root name for output files"}
int    	niter=1     	{prompt="Number to append to output files"}
string	pair="all"	{enum="BC|AD|FE|all",prompt="Pair of images to process (or all)"}
bool	inspect=yes	{prompt="Inspect image lists before proceeding?"}
bool shiftlongaxes=yes	{prompt="Find shifts for image pairs w/overlapping long axes?"}
bool coaddlongaxes=yes	{prompt="Coadd image pairs w/overlapping long axes?"}
string	shortaxesmethod="both"	{enum="xreg|irshift|both|none",prompt="Shift-finding method for overlapping short axes"}
bool use_images=yes {prompt="Measure shifts from images (not file) in irshift?"}
bool display_im=yes	{prompt="Display images for imexam in irshift?"}
bool redo=no		{prompt="Redo shift-finding if already done?"}
bool coaddallimages=yes	{prompt="Coadd all images?"}
bool secondpassxcor=yes	{prompt="Cross-correlate all images with coadded image?"}
bool coaddsecondpass=yes {prompt="Coadd images after second-pass cross-correlation?"}
bool	domkobjmask=yes	{prompt="Make coadded-image object mask (INTERACTIVE)?"}
bool	domasklet=yes	{prompt="Make individual-image object masks?"}
bool	echog=yes	{prompt="Beep when task is done?"}

struct *list
 
begin
 
string in_list, image_root, p_air, ref_image, tmpheaders, sigprefix, rroundname
string theory, method_shortaxes, tempfile2, msect, image_name, tempx, tempy
string zero_out
int xmin, ymin, xint, yint, old_xint, old_yint, xmax_bcad, xmin_fead, ix, iy
int n_iter, xbcad, ybcad, xfead, yfead, bcn1, fen1, xshift, yshift, xe, ye
int xref, yref, int_x, int_y, junkint
int     oxdim,oydim     # dimensions of mask image
int     ixdim,iydim     # dimensions of individual input frames
real xbcad2, ybcad2, xfead2, yfead2, bcad_x, bcad_y, fead_x, fead_y
real xad, yad, xbc, ybc, xfe, yfe, bcad_xlag, bcad_ylag, fead_xlag, fead_ylag
real bcad_xfirstpass, bcad_yfirstpass, bcad_xsecondpass, bcad_ysecondpass
real fead_xfirstpass, fead_yfirstpass, fead_xsecondpass, fead_ysecondpass
real xoff, yoff, step, new_x, new_y, old_x, old_y, step_x, step_y, xsig, ysig
bool in_spect, shift_longaxes, echo_g, coadd_longaxes, abort, go, do_masklet
bool coadd_allimages, xcor_secondpass, coadd_secondpass, do_mkobjmask, re_do
bool im_display, images_use

in_list=inlist
image_root=imageroot
n_iter=niter
p_air=pair

in_spect=inspect
shift_longaxes=shiftlongaxes
re_do=redo
coadd_longaxes=coaddlongaxes
method_shortaxes=shortaxesmethod
images_use=use_images
im_display=display_im
coadd_allimages=coaddallimages
xcor_secondpass=secondpassxcor
coadd_secondpass=coaddsecondpass
do_mkobjmask=domkobjmask
do_masklet=domasklet
echo_g=echog

rroundname=image_root//".xregshifts.shBCADFE"//n_iter
sigprefix=sigmanorm.prefix
go=yes

if(!defpac("imgtools")) {go=no}
if(go) {

# Separate input list into lists of images overlapping along long axes
tmpheaders=mktemp("tmpheaders")
hsel ("@"//in_list, "$I,i_title", yes, >> tmpheaders)
if (!access(in_list//"_AD")) {
 grep ('Position\ \ 1\ of\ \ 6', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_AD")
 grep ('Position\ \ 4\ of\ \ 6', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_AD")
} else {
 print (in_list//"_AD already exists; not created.")
}
if (!access(in_list//"_BC")) {
 grep ('Position\ \ 2\ of\ \ 6', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_BC")
 grep ('Position\ \ 3\ of\ \ 6', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_BC")
} else {
 print (in_list//"_BC already exists; not created.")
}
if (!access(in_list//"_FE")) {
 grep ('Position\ \ 6\ of\ \ 6', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FE")
 grep ('Position\ \ 5\ of\ \ 6', tmpheaders) | fields ("junk", "1", lines="1-9999", >> in_list//"_FE")
} else {
 print (in_list//"_FE already exists; not created.")
}
del(tmpheaders,ver-)
if (in_spect) vi (in_list//"_??")

# Do irshift on lists of images overlapping along long axes
if (shift_longaxes) {
	print ("Finding shifts for images overlapping along long axes...")
	if (p_air=="AD" || p_air=="all") {
		if (re_do || !access(image_root//".roundoff.shAD"//n_iter)) {
			fields (in_list//"_AD", 1, lines="2-2") | scan (ref_image)
			irshift (in_list//"_AD", ref_image, image_root, "shAD"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="BC" || p_air=="all") {
		if (re_do || !access(image_root//".roundoff.shBC"//n_iter)) {
			fields (in_list//"_BC", 1, lines="2-2") | scan (ref_image)
			irshift (in_list//"_BC", ref_image, image_root, "shBC"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
	if (p_air=="FE" || p_air=="all") {
		if (re_do || !access(image_root//".roundoff.shFE"//n_iter)) {
			fields (in_list//"_FE", 1, lines="2-2") | scan (ref_image)
			irshift (in_list//"_FE", ref_image, image_root, "shFE"//n_iter, useimages=images_use, displayim=im_display)
		}
	}
}

# Coadd images overlapping along long axes
if (coadd_longaxes) {
	print ("Coadding images overlapping along long axes...")
	if (p_air=="AD" || p_air=="all") {
		imdel (image_root//"AD"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"AD"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"AD"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"AD"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_AD", image_root//"AD"//n_iter, "", image_root//".roundoff.shAD"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_ABCDEF", makeavg+, flip-)
		sigmanorm (image_root//"AD"//n_iter)
	}
	if (p_air=="BC" || p_air=="all") {
		imdel (image_root//"BC"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"BC"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"BC"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"BC"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_BC", image_root//"BC"//n_iter, "", image_root//".roundoff.shBC"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_ABCDEF", makeavg+, flip-)
		sigmanorm (image_root//"BC"//n_iter)
	}
	if (p_air=="FE" || p_air=="all") {
		imdel (image_root//"FE"//n_iter, ver-, >& "dev$null")
		imdel (image_root//"FE"//n_iter//"em", ver-, >& "dev$null")
		del (image_root//"FE"//n_iter//".imshifts", ver-, >& "dev$null")
		imdel (sigprefix//image_root//"FE"//n_iter, ver-, >& "dev$null")
		ircoadd (in_list//"_FE", image_root//"FE"//n_iter, "", image_root//".roundoff.shFE"//n_iter, "none", "mode", "none", 1, no, "log_"//image_root//"_ABCDEF", makeavg+, flip-)
		sigmanorm (image_root//"FE"//n_iter)
	}
}

# Determine offsets along short axes
#--xreg method------------------------------------------------------------------
if (p_air=="all" && (method_shortaxes=="xreg" || method_shortaxes=="both")) {
    if (!access(image_root//"_shBCAD"//n_iter) || !access(image_root//"_shFEAD"//n_iter)) {
	if (re_do) {
		del (image_root//"_shBCAD"//n_iter//"_rough", ver-, >& "dev$null")
		del (image_root//"_shFEAD"//n_iter//"_rough", ver-, >& "dev$null")
		del (image_root//"_shBCAD"//n_iter//"_1stpass", ver-, >& "dev$null")
		del (image_root//"_shFEAD"//n_iter//"_1stpass", ver-, >& "dev$null")
		del (image_root//"_shBCAD"//n_iter, ver-, >& "dev$null")
		del (image_root//"_shFEAD"//n_iter, ver-, >& "dev$null")
		print ("")
		print ("The imexam log file has been preserved; all else will be recreated.")
	}
	print ("")
	print ("The xregister offset method must be given rough offsets:")
	print ("Find an object in both the BC and AD images.  Get its coords")
	print ("(type 'a' or 'r') in the AD image, then in the BC image.")
	print ("Then find an object in both the FE and AD images, get its coords") 
	print ("in the AD image, then in the FE image, and then type 'q'.")
	print ("The imexam log file will appear for editing in case it's needed.")
	displ (sigprefix//image_root//"BC"//n_iter, 1)
	displ (sigprefix//image_root//"FE"//n_iter, 3)
	displ (sigprefix//image_root//"AD"//n_iter, 2)
	tempfile2=mktemp("temp2")
	print ("#**PROPER FORMAT**: uncommented line 1: AD, 2: BC, 3: AD, 4: FE", >> "log_imexam_"//image_root)
	imexam (sigprefix//image_root//"AD"//n_iter, logfile="log_imexam_"//image_root)
	vi ("log_imexam_"//image_root)
	print ("")
# get rough x,y lags from object pairs (lag=other-AD)
	fields ("log_imexam_"//image_root, "1-2", lines="1-", >> tempfile2)
	fields (tempfile2, "1-2", lines="1-1") | scan (xad,yad)
	fields (tempfile2, "1-2", lines="2-2") | scan (xbc,ybc)
	bcad_xlag=xbc-xad; bcad_ylag=ybc-yad
	print (-1*bcad_xlag,-1*bcad_ylag, >> image_root//"_shBCAD"//n_iter//"_rough")
	print ("Rough shift from "//sigprefix//image_root//"BC"//n_iter//" to "//sigprefix//image_root//"AD"//n_iter//" is "//-1*bcad_xlag//" "//-1*bcad_ylag//" pixels")
	fields (tempfile2, "1-2", lines="3-3") | scan (xad,yad)
	fields (tempfile2, "1-2", lines="4-4") | scan (xfe,yfe)
	fead_xlag=xfe-xad; fead_ylag=yfe-yad
        print (-1*fead_xlag,-1*fead_ylag, >> image_root//"_shFEAD"//n_iter//"_rough")
	print ("Rough shift from "//sigprefix//image_root//"FE"//n_iter//" to "//sigprefix//image_root//"AD"//n_iter//" is "//-1*fead_xlag//" "//-1*fead_ylag//" pixels")
	del (tempfile2, ver-)
# figure out image section limits from rough offsets 
	imgets (sigprefix//image_root//"BC"//n_iter, "i_naxis1"); bcn1=int(imgets.value)
	imgets (sigprefix//image_root//"FE"//n_iter, "i_naxis1"); fen1=int(imgets.value)
	xmax_bcad = bcn1 - int(bcad_xlag)
	xmin_fead = int(abs(fead_xlag))
# first-pass with rough lag
	print ("")
	print ("First pass: immatch.xreg using [1:"//xmax_bcad//",*] xlag="//bcad_xlag//" ylag="//bcad_ylag)
	immatch.xreg (sigprefix//image_root//"BC"//n_iter, sigprefix//image_root//"AD"//n_iter, "[1:"//xmax_bcad//",*]", image_root//"_shBCAD"//n_iter//"_1stpass", datab-, xlag=bcad_xlag, ylag=bcad_ylag, xwindow=25, ywindow=25)
	print ("First pass: immatch.xreg using ["//xmin_fead//":"//fen1//",*] xlag="//fead_xlag//" ylag="//fead_ylag)
	immatch.xreg (sigprefix//image_root//"FE"//n_iter, sigprefix//image_root//"AD"//n_iter, "["//xmin_fead//":"//fen1//",*]", image_root//"_shFEAD"//n_iter//"_1stpass", datab-, xlag=fead_xlag, ylag=fead_ylag, xwindow=25, ywindow=25)
# second-pass with with result of first-pass
	type(image_root//"_shBCAD"//n_iter//"_1stpass") | scan(theory, bcad_xfirstpass, bcad_yfirstpass)
	print ("")
	print ("Second pass: immatch.xreg using [1:"//xmax_bcad//",*] xlag="//-1*bcad_xfirstpass//" ylag="//-1*bcad_yfirstpass)
	immatch.xreg (sigprefix//image_root//"BC"//n_iter, sigprefix//image_root//"AD"//n_iter, "[1:"//xmax_bcad//",*]", image_root//"_shBCAD"//n_iter, datab-, xlag=-1*bcad_xfirstpass, ylag=-1*bcad_yfirstpass, xwindow=25, ywindow=25)
	type(image_root//"_shFEAD"//n_iter//"_1stpass") | scan(theory, fead_xfirstpass, fead_yfirstpass)
	print ("Second pass: immatch.xreg using ["//xmin_fead//":"//fen1//",*] xlag="//-1*fead_xfirstpass//" ylag="//-1*fead_yfirstpass)
	immatch.xreg (sigprefix//image_root//"FE"//n_iter, sigprefix//image_root//"AD"//n_iter, "["//xmin_fead//":"//fen1//",*]", image_root//"_shFEAD"//n_iter, datab-, xlag=-1*fead_xfirstpass, ylag=-1*fead_yfirstpass, xwindow=25, ywindow=25)
	print ("")

# abort if diverging, i.e. if abs(secondpass-firstpass)>0.5*abs(firstpass+rough)
# "+" is because output of xreg is opposite sign of input lag
	abort=no
	type (image_root//"_shBCAD"//n_iter) | scan (theory, bcad_xsecondpass, bcad_ysecondpass)
	if ((abs(bcad_xsecondpass - bcad_xfirstpass)) > (abs(0.5*(bcad_xfirstpass + bcad_xlag)))) abort=yes
	if ((abs(bcad_ysecondpass - bcad_yfirstpass)) > (abs(0.5*(bcad_yfirstpass + bcad_ylag)))) abort=yes
	type (image_root//"_shFEAD"//n_iter) | scan (theory, fead_xsecondpass, fead_ysecondpass)
	if ((abs(fead_xsecondpass - fead_xfirstpass)) > (abs(0.5*(fead_xfirstpass + fead_xlag)))) abort=yes
	if ((abs(fead_ysecondpass - fead_yfirstpass)) > (abs(0.5*(fead_yfirstpass + fead_ylag)))) abort=yes
	if (abort) {
		beep
	  print("WARNING!  xregister did not converge for at least one offset.")
	  print("Stopping task; finish xregister manually and then resume.")
		return
	}
    }
    type (image_root//"_shBCAD"//n_iter) | scan (theory, bcad_xsecondpass, bcad_ysecondpass)
    type (image_root//"_shFEAD"//n_iter) | scan (theory, fead_xsecondpass, fead_ysecondpass)
    if(bcad_xsecondpass>=0)xbcad=int(0.5+bcad_xsecondpass)
    if(bcad_ysecondpass>=0)ybcad=int(0.5+bcad_ysecondpass)
    if(fead_xsecondpass>=0)xfead=int(0.5+fead_xsecondpass)
    if(fead_xsecondpass>=0)yfead=int(0.5+fead_ysecondpass)
    if(bcad_xsecondpass<0)xbcad=int(-0.5+bcad_xsecondpass)
    if(bcad_ysecondpass<0)ybcad=int(-0.5+bcad_ysecondpass)
    if(fead_xsecondpass<0)xfead=int(-0.5+fead_xsecondpass)
    if(fead_xsecondpass<0)yfead=int(-0.5+fead_ysecondpass)
}

#--irshift method---------------------------------------------------------------
if (p_air=="all" && (method_shortaxes=="irshift" || method_shortaxes=="both")) {
    if(!access(image_root//".roundoff.shBCAD"//n_iter) || !access(image_root//".roundoff.shFEAD"//n_iter)) {
        if (re_do) {
		del(image_root//".imshifts.sh??"//n_iter, ver-, >& "dev$null")
		del(image_root//".offsets.sh??"//n_iter, ver-, >& "dev$null")
		del(image_root//".roundoff.sh??"//n_iter, ver-, >& "dev$null")
		del(image_root//".shiftexact.sh??"//n_iter, ver-, >& "dev$null")
		del(image_root//".shiftfrac.sh??"//n_iter, ver-, >& "dev$null")
		del(image_root//".shiftint.sh??"//n_iter, ver-, >& "dev$null")
		print ("The *allcoords* and *refcoords* files have been saved;")
		print ("all else will be recreated.")
	}
	if(!access(in_list//"_BCAD")) {
		print (sigprefix//image_root//"BC"//n_iter, >> in_list//"_BCAD")
		print (sigprefix//image_root//"AD"//n_iter, >> in_list//"_BCAD")
	}
	irshift (in_list//"_BCAD", sigprefix//image_root//"AD"//n_iter, image_root, "shBCAD"//n_iter, useimages=images_use, displayim=im_display)
	if(!access(in_list//"_FEAD")) {
		print (sigprefix//image_root//"FE"//n_iter, >> in_list//"_FEAD")
		print (sigprefix//image_root//"AD"//n_iter, >> in_list//"_FEAD")
	}
	irshift (in_list//"_FEAD", sigprefix//image_root//"AD"//n_iter, image_root, "shFEAD"//n_iter, useimages=images_use, displayim=im_display)
    }
    fields (image_root//".shiftexact.shBCAD"//n_iter, "1-2", lines="1-1") | scan (xbc, ybc)
    fields (image_root//".shiftexact.shBCAD"//n_iter, "1-2", lines="2-2") | scan (xad, yad)
    fields (image_root//".shiftexact.shFEAD"//n_iter, "1-2", lines="1-1") | scan (xfe, yfe)
    fields (image_root//".shiftexact.shFEAD"//n_iter, "1-2", lines="2-2") | scan (xad, yad)
    xbcad2=xbc-xad; ybcad2=ybc-yad; xfead2=xfe-xad; yfead2=yfe-yad

# set xbcad ybcad xfead yfead if "both"
    if (method_shortaxes=="both") {
	print ("")
print ("BC/AD xregister offsets: "//bcad_xsecondpass//" "//bcad_ysecondpass)
	print ("BC/AD irshift   offsets: "//xbcad2//" "//ybcad2)
	tempx = mktemp("tempx"); tempy = mktemp("tempy")
	print (xbcad, >> tempx); print (xbcad2, >> tempx)
	print (ybcad, >> tempy); print (ybcad2, >> tempy)
	type (tempx) | average | scan (bcad_x, xsig, junkint)
	type (tempy) | average | scan (bcad_y, ysig, junkint)
  print ("BC/AD average offsets: "//bcad_x//"+-"//xsig//" "//bcad_y//"+-"//ysig)
	del (tempx, ver-); del (tempy, ver-)
	if(!access(image_root//"_shBCAD"//n_iter//"_irshift"))print(xbcad2, ybcad2, >> image_root//"_shBCAD"//n_iter//"_irshift")
	if(!access(image_root//"_shBCAD"//n_iter//"_xreg"))copy(image_root//"_shBCAD"//n_iter,image_root//"_shBCAD"//n_iter//"_xreg")
	print (bcad_x, bcad_y, xsig, ysig, >> image_root//"_shBCAD"//n_iter)
	print ("")
print ("FE/AD xregister offsets: "//fead_xsecondpass//" "//fead_ysecondpass)
	print ("FE/AD irshift   offsets: "//xfead2//" "//yfead2)
	tempx = mktemp("tempx"); tempy = mktemp("tempy")
	print (xfead, >> tempx); print (xfead2, >> tempx)
	print (yfead, >> tempy); print (yfead2, >> tempy)
	type (tempx) | average | scan (fead_x, xsig, junkint)
	type (tempy) | average | scan (fead_y, ysig, junkint)
  print ("FE/AD average offsets: "//fead_x//"+-"//xsig//" "//fead_x//"+-"//ysig)
	del (tempx, ver-); del (tempy, ver-)
	if(!access(image_root//"_shFEAD"//n_iter//"_irshift"))print(xfead2, yfead2, >> image_root//"_shFEAD"//n_iter//"_irshift")
	if(!access(image_root//"_shFEAD"//n_iter//"_xreg"))copy(image_root//"_shFEAD"//n_iter,image_root//"_shFEAD"//n_iter//"_xreg")
	print (fead_x, fead_y, xsig, ysig, >> image_root//"_shFEAD"//n_iter)
	print ("")
	xbcad=int(abs(0.5+bcad_x)); ybcad=int(abs(0.5+bcad_y))
	xfead=int(abs(0.5+fead_x)); yfead=int(abs(0.5+fead_y))
    }
}

# coadd all images at once
if (p_air=="all" && coadd_allimages) {
# set minimum x,y offsets to be 0 & 0 (independently)
	if (!access(image_root//".roundoff.shAD"//n_iter//"min0")) {
		list=image_root//".roundoff.shAD"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shAD"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shAD"//n_iter//"min0")
	}
	if (!access(image_root//".roundoff.shBC"//n_iter//"min0")) {
		list=image_root//".roundoff.shBC"//n_iter
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shBC"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shBC"//n_iter//"min0")
	}
	list=image_root//".roundoff.shFE"//n_iter
	if (!access(image_root//".roundoff.shFE"//n_iter//"min0")) {
		xmin=9999; ymin=9999
		while (fscan(list,xint,yint) !=EOF) {
			if (xint < xmin) xmin = xint
			if (yint < ymin) ymin = yint
		}
		filecalc(image_root//".roundoff.shFE"//n_iter, "$1-"//xmin//";$2-"//ymin, format="%4.0f %4.0f", >> image_root//".roundoff.shFE"//n_iter//"min0")
	}

# then offset by BC/AD or FE/AD offsets, as appropriate
	if(!access(image_root//".roundoff.shBCADFE"//n_iter)) {
		filecalc (image_root//".roundoff.shBC"//n_iter//"min0", "$1+"//xbcad//";$2+"//ybcad, format="%4.0f %4.0f", >> image_root//".roundoff.shBCADFE"//n_iter)
		type (image_root//".roundoff.shAD"//n_iter//"min0", >> image_root//".roundoff.shBCADFE"//n_iter)
		filecalc (image_root//".roundoff.shFE"//n_iter//"min0", "$1+"//xfead//";$2+"//yfead, format="%4.0f %4.0f", >> image_root//".roundoff.shBCADFE"//n_iter)
	}
	if(!access(in_list//"_BCADFE")) concat (in_list//"_BC,"//in_list//"_AD,"//in_list//"_FE", in_list//"_BCADFE")

# do the coadd 
	print ("Coadding all images at once...")
	ircoadd (in_list//"_BCADFE", image_root//"BCADFE"//n_iter//"_1stpass", "", image_root//".roundoff.shBCADFE"//n_iter, "none", "mode", "none", 1, no, "log_"//in_list//"_ABCDEF", stdpars-, makeavg+, flip-, imcoaddpars="phiirs$ircoaddonis.par")
}

# cross-correlate individual images with coadded image
if (p_air=="all" && xcor_secondpass) {
	print ("Creating constant-RMS coadded image...")
	ixdim=512; iydim=1024
	if (!access(sigprefix//image_root//"BCADFE"//n_iter//"_1stpass.fits")) sigmanorm (image_root//"BCADFE"//n_iter//"_1stpass.fits")
	print ("Cross-correlating individual images with coadded image...")
	imgets(sigprefix//image_root//"BCADFE"//n_iter//"_1stpass","i_naxis1")
	oxdim=int(imgets.value)
	imgets(sigprefix//image_root//"BCADFE"//n_iter//"_1stpass","i_naxis2")
	oydim=int(imgets.value)
	list=image_root//"BCADFE"//n_iter//"_1stpass.imshifts"
	while (fscan(list,image_name,xshift,yshift) !=EOF) {
# zero out bad pixels if not already done
                imgets(image_name,"ZEROOUT")
                zero_out=imgets.value
                if(zero_out=="0"){
                        imgets(image_name,"BPMASK")
                        if(imgets.value!="0"){
imar(image_name,"*",imgets.value,image_name,pixt="real",calct="real",ver-,noac-)
hedit(image_name,"ZEROOUT",imgets.value,add+,ver-,show-,update+)
                        } else {
print("Bad pixels not zeroed for "//image_name//", and no BP mask in header!")
                        }
                }
		ix=xshift+1-14; if(ix<1)ix=1
		iy=yshift+1-14; if(iy<1)iy=1
        	xe=xshift + ixdim + 14; if(xe>oxdim)xe=oxdim
		ye=yshift + iydim + 14; if(ye>oydim)ye=oydim
        	msect = "["//ix//":"//xe//","//iy//":"//ye//"]"
		print ("Rough shift from "//image_name//" to "//sigprefix//image_root//"BCADFE"//n_iter//"_1stpass is "//xshift//" "//yshift//" pixels")
		immatch.xreg (image_name, sigprefix//image_root//"BCADFE"//n_iter//"_1stpass", msect, image_root//"BCADFE"//n_iter//"_xregout", datab+, records="", xlag=-1*xshift, ylag=-1*yshift, xwindow=25, ywindow=25, append+)
	}
	tempx = mktemp("tempx"); tempy = mktemp("tempy")
	grep ('xshift', image_root//"BCADFE"//n_iter//"_xregout") | fields ("junk", "2", >> tempx)
	grep ('yshift', image_root//"BCADFE"//n_iter//"_xregout") | fields ("junk", "2", >> tempy)
	if(!access(image_root//".xregexact.shBCADFE"//n_iter)) join (tempx, tempy, output=image_root//".xregexact.shBCADFE"//n_iter)
	del (tempx, ver-); del (tempy, ver-)
# Create optimum rounded-off integer offsets
	print(" ")
	print("===) Finding optimum second-pass rounded-off integer offsets...")
	old_x = 999.; old_y = 999.
# for each 0.01 pixel step...
	for (i=-50; i<50; i+=1) {
		step = i / 100.
#		print (i)
# ...recalculate integer offsets...
		list=image_root//".xregexact.shBCADFE"//n_iter
		del (rroundname, ver-, >& "dev$null")
        	while(fscan(list,xoff,yoff) !=EOF) {
			xoff=xoff+step
			yoff=yoff+step
                	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                        	print(xoff,yoff, >> "temp."//rroundname)
                	} else {
                        	if((abs(xoff))<=0.5) {
                                	xoff=0.
                        	}
                        	if((abs(yoff))<=0.5) {
                                	yoff=0.
                        	}
                        	print(xoff,yoff, >> "temp."//rroundname)
                	}
     filecalc("temp."//rroundname, "$1;$2", format="%4.0f %4.0f", >> rroundname)
     			flpr; flpr; flpr
                	del ("temp."//rroundname, ver-, >& "dev$null")
        	}
        	del ("temp."//rroundname, ver-, >& "dev$null")
# calculate sum of squares of residuals of exact offsets - new integer offsets
		filecalc (image_root//".xregexact.shBCADFE"//n_iter//","//rroundname, "$1@1-$1@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_x=average.sumsqrs
		filecalc (image_root//".xregexact.shBCADFE"//n_iter//","//rroundname, "$2@1-$2@2+"//step) | average (option = "new_sample", >& "dev$null")
		flpr; flpr; flpr
		new_y=average.sumsqrs
		if (new_x < old_x) { 
			old_x = new_x
			step_x = step
		}
		if (new_y < old_y) {
			old_y = new_y
			step_y = step
		}
	}
# recalculate final integer offsets
	list=image_root//".xregexact.shBCADFE"//n_iter
	del(rroundname,ver-)
	del ("temp."//rroundname, ver-, >& "dev$null")
	while(fscan(list,xoff,yoff) !=EOF) {
		xoff=xoff+step_x
		yoff=yoff+step_y
        	if(((abs(xoff))>=0.5) && ((abs(yoff))>=0.5)) {
                	print(xoff,yoff, >> "temp."//rroundname)
        	} else {
                	if((abs(xoff))<=0.5) {
                       	xoff=0.
                	}
                	if((abs(yoff))<=0.5) {
                       	yoff=0.
                	}
                	print(xoff,yoff, >> "temp."//rroundname)
        	}
        filecalc("temp."//rroundname,"$1;$2",format="%4.0f %4.0f", >>rroundname)
		flpr; flpr; flpr
        	del ("temp."//rroundname, ver-, >& "dev$null")
	}
	del ("temp."//rroundname, ver-, >& "dev$null")
}

# ** next 3 can be done in dophiirs

# Coadd images after second-pass cross-correlation
if (p_air=="all" && coadd_secondpass) {
	print ("Coadding after second-pass cross-correlation...")
	ircoadd(in_list//"_BCADFE", image_root//"BCADFE"//n_iter, "", rroundname, "none", "mode", "none", 1, no, "log_"//in_list//"_ABCDEF", stdpars-, makeavg-, flip-, imcoaddpars="phiirs$ircoaddonis.par")
}

# Make coadded-image object mask
if (p_air=="all" && do_mkobjmask) {
	print ("Make coadded-image object mask...")
	mkobjmask (image_root//"BCADFE"//n_iter//".fits", "", suffix="objmask", nsmooth=3, subsample=3, filtsize=15, ngrow=1, interact+)
}

# Make individual-image object masks
if (p_air=="all" && do_masklet) {
	print ("Make individual-image object masks...")
	masklet (image_root//"BCADFE"//n_iter, "", "om", shiftfile="", editfile+, flip-, update+)
}

} else {
	print ("WARNING! Package stsdas.imgtools not loaded... task aborted.")
}

if (echo_g) beep

end

