# onisfirst.cl -- task to subtract dark & add header params to ONIS images
#
# 1. subtract dark
# 2. hedit image headers, adding: (basically this task is hadd.cl plus #0)
# 	1. BPM,BPMASK = bad pixel mask name
#	2. GAIN,RDNOISE if not present (currently assumes 4.,35.)
#	3. coadds       = number of coadds (copy of ncoadds)
# 	4. itime,exptime  = exposure time per coadd (old exptime)
#				and total exposure time (itime * coadds)
# 	5. obs-date = date-obs but in dd/mm/yy format
# 	6. ut       = time-obs (or time-obs + 7)
# 	7. date-obs = yyyy-mm-dd format (old date-obs saved as dateorig)
# (following are optional)
# 	8. epoch, ra, dec = entered as parameters, if desired
# 	9. st = sidereal time, calculated from ut & date using astutil.asttimes
# 		& the observatory parameter (kpno, mtbigelow, mmto, ctio)
# 	10. airmass  = calculated from epoch, ra, dec, st, and obs-date
#
# 990202: created ONIS version; original exptime stored as itime
# 990320: fixed bug where exptime & itime were feedbacking on each other
# 990420: now adds gain & rdnoise to image header if not already there
# 990427: DARKSUB keyword added to darksubbed images, not original ones
# 990523: '>& "dev$null"' search for DARKSUB keyword to suppress error msg
#
# **bug: airmass+ airauto+ option assumes you're observing from MST time zone!
#	...but that's only for the "asttimes" task, which may be irrelevant
# **should be more options for computing ut than just time-obs or time-obs+7h

procedure onisfirst(imagelist,darksub,badmask)

string imagelist  {prompt="List of images to process (do not use an `@')"}
bool  darksub=yes {prompt="Subtract off dark frame?"}
string badmask    {prompt="Name of bad pixel mask to add to image header"}

string dark       {prompt="Name of the dark frame to subtract", mode="q"}
string darkstring {prompt="String to append to dark-subtracted images",mode="q"}
bool   coords=no  {prompt="Add RA, DEC, and EPOCH to image header?"}
bool  airmass=yes {prompt="Enter airmass into image header?"}
bool  airauto=yes {prompt="Find airmass of image automatically?"}
real   airval     {prompt="Airmass of image", mode="q"}
string epoch="1950.0" {prompt="Epoch of coordinates"}
string ra=""	  {prompt="Right Ascension (surround with quotes)"}
string dec=""	  {prompt="Declination (surround w/quotes; use - but not +"}
string observatory="kpno" {prompt="Observatory where observations were made"}
bool   ut=yes      {prompt="Is 'time-obs' in header ut or not?"}
bool   ddmmyy=yes  {prompt="Is 'date-obs' in day/month/year format?"}
bool   echog       {yes,prompt="Beep when task is done?"}
struct *list

begin

string ddark, ddarkstring, bbadmask, check, base, iimagelist, ext
string darklist, flat, flatbase, meanfile, currentflat, tempflat, name
string editlist, current, obsdate, rra, ddec, eepoch, oobservatory, minsec
string obstime, timeobs, dateobs, century
real intime, aairval, ttime, mean, meanbad, x
int ncoadd, ddy, hhr, hhour, mmonth, yyear, dday, uttime, last2digits
bool ddarksub, ccoords, aairauto, uut, aairmass

iimagelist = imagelist
list = iimagelist
editlist = iimagelist
ddarksub = darksub
if (ddarksub) {
	ddark = dark
	ddarkstring = darkstring
	darklist = iimagelist//ddarkstring
}
bbadmask = badmask
ccoords = coords
aairmass = airmass
if (aairmass) {
	aairauto = airauto
#	if(!aairauto)aairval = airval
	if(!defpac("astutil")) {
		beep
  		print ("WARNING!!  Package ''astutil'' is not loaded; cannot set airmass in images.")
  		print ("Load the package and rerun the task.")
		return
		}
}
eepoch = epoch
rra = ra
ddec = dec
oobservatory = observatory
uut = ut

# Add BPM and other goodies to the image headers, just like in the hadd.cl task
print("===> Adding various parameters to the image headers...")
list = iimagelist
while (fscan (list, current) !=EOF) {
  hedit(current,"BPM",bbadmask,add+,ver-,show+,update+)
  hedit(current,"BPMASK",bbadmask,add+,ver-,show+,update+)
# Currently assumes gain=4, rdnoise=35
  imgets(current,"GAIN", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"GAIN","4.",add+,ver-,show+,updat+)
  }
  imgets(current,"RDNOISE", >& "dev$null")
  if (imgets.value=="0" || imgets.value=="") {
    hedit(current,"RDNOISE","35.",add+,ver-,show+,updat+)
  }
# Set coadds=(# of coadds)
  imgets(current,"ncoadds", >& "dev$null")
  ncoadd=real(imgets.value)
  hedit(current,"coadds",(ncoadd),add+,ver-,show+,update+)
# Set itime=exposure time per coadd
  imgets(current,"itime", >& "dev$null")
  if (imgets.value!="0") {
    print ("ITIME keyword already exists -- onisfirst already run on image?")
  } else {
    imgets(current,"exptime", >& "dev$null")
    intime=real(imgets.value)
    hedit(current,"itime",(intime),add+,ver-,show+,update+)
# Set exptime=itime * (# of coadds)
    hedit(current,"exptime",(ncoadd*intime),add+,ver-,show+,update+)
  }
#
# The 'date-obs' keyword comes in either the old format "dd/mm/yy",
# the erroneous format "mm/dd/yy", or the new format "yyyy-mm-dd".
# Create new 'date-obs' keyword in the new format "yyyy-mm-dd".
  imgets(current,"dateorig",>&"dev$null")
  if (imgets.value!="0") {
    print ("DATEORIG keyword already exists -- onisfirst already run on image?")
  } else {
    imgets(current,"date-obs")
# Copy 'date-obs' keyword to 'dateorig' in original format.
    hedit(current,"dateorig",imgets.value,add+,ver-,show+,update+)
# Create header keyword 'obs-date', always in "dd/mm/yy" format.
    if(!ddmmyy){
# mm/dd/yy format:
      if(substr(imgets.value,3,3)=="/") {
        obsdate=substr(imgets.value,4,6)//substr(imgets.value,1,3)//substr(imgets.value,7,8)
      } else {
# yyyy-mm-dd format:
#  1234567890
        obsdate=substr(imgets.value,9,10)//"/"//substr(imgets.value,6,7)//"/"//substr(imgets.value,3,4)
      }
# create obs-date if date-obs in mm/dd/yy or yyyy-mm-dd format:
      hedit(current,"obs-date",obsdate,add+,ver-,show+,update+)
    } else {
# create obs-date if date-obs in dd/mm/yy format:
      hedit(current,"obs-date",imgets.value,add+,ver-,show+,update+)
    }
# Create new 'date-obs' in new format "yyyy-mm-dd" from 'obs-date' "dd/mm/yy"
#								    12345678
    imgets(current,"obs-date")
    if ((substr(imgets.value,7,7))=="9") {
  	  century=19
    } else {
  	  century=20
    }
    dateobs=century//substr(imgets.value,7,8)//"-"//substr(imgets.value,4,5)//"-"//substr(imgets.value,1,2)
    hedit(current,"date-obs",dateobs,add+,ver-,show+,update+)
#
# Sometimes the time in the image header is ut, sometimes not.
# so this option lets you choose either ut or mst
  if(uut){hedit(current,"ut",("(@'time-obs')"),add+,ver-,show+,update+)}
  else{
    imgets(current,"time-obs")
    obstime=substr(imgets.value,1,2)
    uttime=int(obstime)+7
    if(uttime>=24){uttime=uttime-24}
    if(uttime<=9){timeobs="0"//str(uttime)//substr(imgets.value,3,8)}
    if(uttime>=10){timeobs=str(uttime)//substr(imgets.value,3,8)}
#need to hedit ut twice cause is not in hh:mm:ss format 1st time around
    hedit(current,"ut",timeobs,add+,ver-,show+,update+)
    hedit(current,"ut",timeobs,add+,ver-,show+,update+)
    }
  }
}

if(ccoords){
	list = iimagelist
	while (fscan (list, current) !=EOF) {
  		hedit(current,"epoch",eepoch,add+,ver-,show+,update+)
  		hedit(current,"ra",rra,add+,ver-,show-,update+)
  		hedit(current,"ra",rra,add+,ver-,show+,update+)
  		hedit(current,"dec",ddec,add+,ver-,show-,update+)
  		hedit(current,"dec",ddec,add+,ver-,show+,update+)
#hedit test ra "(str('21:51:59.0'))" add+ ver- show-
	}
}

if(aairmass){
  if(aairauto){
    list = iimagelist
    while (fscan (list, current) !=EOF) {
  	imgets(current,"obs-date")
  	dday=int(substr(imgets.value,1,2))
  	mmonth=int(substr(imgets.value,4,5))
#  	yyear=int("19"//substr(imgets.value,7,8))
	last2digits=int(substr(imgets.value,7,8))
	if(last2digits <= 80) {
  	  yyear=int("20"//substr(imgets.value,7,8))
	} else {
  	  yyear=int("19"//substr(imgets.value,7,8))
	}
  	imgets(current,"ut")
  	hhr=int(substr(imgets.value,1,2))
  	minsec=substr(imgets.value,3,8)
	hhour=hhr-7
	if(hhour<0){
		dday=dday-1
		hhour=hhour+24.
	}
	if(dday<=0){
		mmonth=mmonth-1
# this will not always be totally accurate, and may fail for feb., but hey.
		dday=30
	}
# observing on new year's eve will require a new correction!
	ttime=real(str(int(hhour))//minsec)
	asttimes(observatory=oobservatory, year=yyear, month=mmonth, day=dday, time=ttime)
   	hedit(current,"st",asttimes.lmst,add+,ver-,show-,update+)
   	hedit(current,"st",asttimes.lmst,add+,ver-,show+,update+)
	setairmass(current, observatory=oobservatory, intype="middle", outtype="effective", date="obs-date",exposure="exptime", utmiddle="utmiddle", show+, update+, override+)
    }
  } else {
	list = iimagelist 
	while (fscan (list, current) !=EOF) { 
		print("Setting airmass for image "//current)
		hedit(current,"airmass",airval,add+,ver-,show+,update+)
	}
  }
}

# Subtract off Dark Current 
if (ddarksub) {
	list = iimagelist 
# if darklist doesn't exist, assume dsub imgs don't either; create it & dsubs
	if(!access(darklist)) {
	    print("===> Subtracting off Dark Current...")
	    while (fscan (list, name) !=EOF) {
		fileroot(name,validim+)
		base=fileroot.root
		ext=fileroot.extension
		if(ext!="")ext="."//ext
		print(base//ddarkstring//ext, >> darklist)
  		imgets(name,"DARKSUB",>&"dev$null")
		if (imgets.value=="0") {
		    imarith(name, "-", ddark, base//ddarkstring//ext, verbose-)
	   hedit(base//ddarkstring//ext,"DARKSUB",ddark,add+,ver-,show+,update+)
		} else {
  print("DARKSUB keyword exists; onisfirst already run? "//name//" just copied")
		    imcopy(name, base//ddarkstring//ext, verbose-)
		}
	    }
	} else {
	    print("Darksubbed image list "//darklist//" already exists; images assumed to exist too.")
	}
#imarith("@"//iimagelist, "-", ddark, "@"//darklist, verbose-)
#hedit("@"//darklist,"DARKSUB",ddark,add+,ver-,show+,update+)
print(" ") 
print("===> Dark-subtracted images are listed in the file "//darklist//".") 
}

if(echog)beep

end

