# awk script used by the PHIIRS routine photscale to measure photometric
# variations from frame to frame over an observing sequence.   This routine 
# uses the formatted output from PHIIRS routine photcheck + awk script 
# photcheck.awk as its input.  
#
# This version deals with aperture corrections in col 1, image name and 
# airmass in cols 2 & 3, and mag,err pairs for each reference star 
# in even and odd numbered columns thereafter.
#
# The first line of the input file is considered to be the photometry for the
# "reference frame."  Magnitudes of stars in each subsequent line are compared 
# to it, and the average and rms of the magnitude differences is output in both
# magnitude and flux form, along with the individual stellar magnitude 
# differences.  Magnitude and error records that are INDEF are ignored.
#
# originally created 3 August 1993 by Mark Dickinson
# modified March 1996 by Pat Hall
#
{
#
#   Record magnitudes from first line as reference values.
#
	if (NR==1) {
		ii=0
		nn=0
		for (i=4;i<=NF;i+=2) {
			ii+=1
			if ($i == "INDEF") {
				m[ii] = -99.99
				v[ii] = -99.99
			   } else {
				nn+=1
				m[ii] = $i + $1
				v[ii] = $(i+1)*$(i+1)
			}
		}
		printf ("%26s %7s %7s %6s %7s %6s %2g ",$2,"airmass","delta_m"," err_m","scaling"," err_s",nn)
		ii=0
		for (i=4;i<=NF;i+=2) {
			ii+=1
			if ($i == "INDEF") {
				printf (" %6s", " INDEF")
			   } else {
				printf (" %6.3f", m[ii])
			}
		}
		printf ("\n")
#
#   For all other lines, calculate differences between the star magnitudes and
#   the reference values, print these out, and calculate the mean and rms of 
#   those differences, then print those out at the end of the line.   Stars with
#   magnitude values = INDEF are excluded from the mean and rms calculation.
#
	   } else {
		printf ("%26s %7.4f ",$2,$3)
		dsum = 0.
		wsum = 0.
		nn = 0
		ii=0
		for (i=4;i<=NF;i+=2) {
			ii+=1
			if ($i != "INDEF" && m[ii] != -99.99) {
				d[ii] = m[ii] - $i - $1
				vd[ii] = $(i+1)*$(i+1) + v[ii]
				dsum += d[ii]/vd[ii]
				wsum += 1./vd[ii]
				nn += 1
			   } else {
				d[ii] = -99.99
			}
		}
		if (nn > 0) {
			dmean = dsum / wsum
			derr  = sqrt(1./wsum)
#
#   New mean & error in flux, not magnitude, units
#
#			nmean = exp(log(10.0)*dmean/2.5)
#
			nmean = exp(log(10.0)*dmean/-2.5)
			nerr = 0.4*log(10.0)*nmean*derr
			printf ("%7.3f %6.3f %7.3f %6.3f %2g ", dmean, derr, nmean, nerr, nn)
		   } else {
			printf ("%7s %6s %7s %6s %2g ","  INDEF"," INDEF","  INDEF"," INDEF", 0)
		}
		ii=0
		for (i=4;i<=NF;i+=2) {
			ii+=1
			if ($i != "INDEF" && m[ii] != -99.99) {
				printf (" %6.3f", d[ii])
			   } else {
				printf (" %6s"," INDEF")
			}
		}
		printf ("\n")
	}
}
# i	column
# ii	star counter
# nn	good star counter
# m[ii]	magref
# v[ii] varref
# d[ii] magref - mag
# vd[ii] varref + var
# dsum  sum of d[]
# wsum	sum of vd[]
