# trim -- create constant-rms image with a certain minimum exposure time
#
# This task takes an input image and exposure map and creates:
#
# "trim"//img	   	trimmed image
# "trim"//mask     	exposure map for trimmed image
# "trim"//mask//"mask"  exposed/unexposed area mask for trimmed image
# stats			files of trimmed image RMS values (from maskstat)
#
# 960710: created from batch routine dofocasprep.cl
# 990307: added subtract option
# 990515: when appropriate, use imcalc not imarith

procedure trim(image,mask,stats)

string image 	{prompt="Image (or image list) to trim"}
string mask  	{prompt="Exposure map or list (default=image//em)"}
string stats 	{prompt="File to contain trimmed image RMS values"}

string out="trim"    {prompt="Prefix for trimmed images"}
bool   subtract=no   {prompt="Subtract average from image before making mask?"}
real mingoodexp=0.36 {prompt="Minimum acceptable exptime (fraction of maximum)"}
bool   echog         {yes,prompt="Beep when task is done?"}
struct *list

begin

string img, exp, newexp, pprefix, trimlist, imglist, trimmask, explist
string sstats, joinlist, checki, checke, ext, tmpsub
real mmingoodexp
bool doren, sub_tract

img = image
	checki=substr(img,1,1)
	if(checki=="@"){
		imglist=substr(img,2,strlen(img))
		trimlist=imglist//"trim"
	} else {
		imglist=mktemp("imglist")
		trimlist=imglist//"trim"
		print (img, >> imglist)
	}
exp = mask
	checke=substr(exp,1,1)
	if(checke=="@"){
		explist=substr(exp,2,strlen(exp))
	} else {
		explist=mktemp("explist")
		if(checke=="") {
			fileroot (img, validim+)
			print (fileroot.root//"em", >> explist)
		} else {
                	print (exp, >> explist)
		} 
	} 
sstats=stats

pprefix=out
	if(pprefix=="")pprefix="trim"
sub_tract=subtract
mmingoodexp=mingoodexp

# join image and mask lists
joinlist=mktemp("joinlist")
join (imglist, explist, >> joinlist)
list=joinlist

while (fscan (list,img,exp) != EOF) {
	fileroot(img, validim+)
	img = fileroot.root
	fileroot(exp, validim+)
	exp = fileroot.root
	ext = fileroot.extension
	if(ext!="") ext="."//ext
	newexp = pprefix//fileroot.root
# the core
	if (!access(newexp//ext)) {
    print ("Preparing exposure map...")
		minmax (exp, force+, update+, ver-)
		imar (exp, "/", minmax.maxval, newexp, pixt="real", ver-, noac-)
		#imreplace (newexp, 0., lower=INDEF, upper=mmingoodexp)
		#imreplace (newexp, 1., lower=mmingoodexp-0.01, upper=INDEF)
		imcalc (newexp, newexp, "if im1 .le. "//mmingoodexp//" then 0 else 1", ver-)
		addring (newexp, newexp, ngrow=5, badval=1)
		imar (newexp, "*", exp, newexp, ver-, pixt="real", noact-)
	}
	if (!access(pprefix//img//"mask.pl")) {
    print ("Creating trimmed area mask...")
		#imcopy (newexp, pprefix//img//"mask.pl", ver-)
		#imreplace (pprefix//img//"mask.pl",1.,lower=0.01,upper=INDEF)
		imcalc (newexp, pprefix//img//"mask.pl", "if im1 .eq. 0 then 0 else 1", ver-)
	}
# subtraction phase
	if (sub_tract) {
    print ("Subtracting mean from image...")
		tmpsub=mktemp("tmpsub")
        	maskstat(img,pprefix//img//"mask.pl",1.,nsigrej=5.,maxiter=5.,lower=INDEF,upper=INDEF,ver-,iterstat+,print-,addh-)
         	imar(img,"-",iterstat.mean,tmpsub,calctype="real",ver-,noact-)
    print ("Normalizing image...")
		sigmanorm (tmpsub, normim=newexp, prefix=pprefix)
		imren (pprefix//tmpsub, pprefix//img, ver-)
		imdel (pprefix//tmpsub, ver-, >& "dev$null")
		imdel (tmpsub, ver-, >& "dev$null")
	} else {
    print ("Normalizing image...")
		sigmanorm (img, normim=newexp, prefix=pprefix)
	}
	print (pprefix//img, >> trimlist)
    print ("Getting statistics...")
	maskstat (pprefix//img, pprefix//img//"mask.pl", goodv=1, iterstat+, ver-, >> sstats)
}

del (joinlist, ver-)
if (checke!="@") del (explist, ver-)
if (checki!="@") {
	del (imglist, ver-)
	del (trimlist, ver-)
} else {
	print("Output trimmed images are listed in "//trimlist)
}
print("Output trimmed image statistics are listed in "//sstats)

#imar cf1.coadd3em / 3879.999267578125 cf1.coadd3em2
#imreplace cf1.coadd3em2 0. lower=INDEF upper=0.4
#imreplace cf1.coadd3em2 1. lower=0.39 upper=INDEF
#addring cf1.coadd3em2 cf1.coadd3em2 ngrow=5 badval=1
#imar cf1.coadd3em * cf1.coadd3em2 cf1.coadd3em2
#imar cf1.coadd3em2 / 3879.999267578125 cf1.coadd3em2
#sigmanorm cf1.coadd3 normim=cf1.coadd3em2
#imcopy cf1.coadd3em2.imh cf1.coadd3em3.imh
#imreplace cf1.coadd3em3.imh 1. lower=0.10 upper=INDEF
#imren normcf1.coadd3.imh cf1
#maskstat cf1.imh cf1.coadd3em3.imh goodv=1 iterstat+ ver- >> stats

if(echog)beep

end

